@@ -14,19 +14,19 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
    */ 
 
 #ifndef __NSComparisonPredicate_h_GNUSTEP_BASE_INCLUDE
 #define __NSComparisonPredicate_h_GNUSTEP_BASE_INCLUDE
 #import	<GNUstepBase/GSVersionMacros.h>
 
-#if	OS_API_VERSION(100400, GS_API_LATEST)
+#if	OS_API_VERSION(MAC_OS_X_VERSION_10_4, GS_API_LATEST)
 
 #import	<Foundation/NSExpression.h>
 #import	<Foundation/NSPredicate.h>
@@ -39,13 +39,13 @@ typedef enum _NSComparisonPredicateModifier
 {
   NSDirectPredicateModifier=0,
   NSAllPredicateModifier,
-  NSAnyPredicateModifier,
+  NSAnyPredicateModifier
 } NSComparisonPredicateModifier;
 
 typedef enum _NSComparisonPredicateOptions
 {
   NSCaseInsensitivePredicateOption=0x01,
-  NSDiacriticInsensitivePredicateOption=0x02,
+  NSDiacriticInsensitivePredicateOption=0x02
 } NSComparisonPredicateOptions;
 
 typedef enum _NSPredicateOperatorType
@@ -62,13 +62,14 @@ typedef enum _NSPredicateOperatorType
   NSEndsWithPredicateOperatorType,
   NSInPredicateOperatorType,
   NSCustomSelectorPredicateOperatorType
-#if OS_API_VERSION(100500,GS_API_LATEST) 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5,GS_API_LATEST) 
   ,
   NSContainsPredicateOperatorType = 99,
   NSBetweenPredicateOperatorType
 #endif
 } NSPredicateOperatorType;
 
+GS_EXPORT_CLASS
 @interface NSComparisonPredicate : NSPredicate
 {
 #if	GS_EXPOSE(NSComparisonPredicate)
