@@ -14,12 +14,12 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
    */
 
 #ifndef __NSDistributedNotificationCenter_h_GNUSTEP_BASE_INCLUDE
@@ -78,6 +78,7 @@ GS_EXPORT NSString* const GSPublicNotificationCenterType;
 GS_EXPORT NSString* const GSNetworkNotificationCenterType;
 #endif
 
+GS_EXPORT_CLASS
 @interface	NSDistributedNotificationCenter : NSNotificationCenter
 {
 #if	GS_EXPOSE(NSDistributedNotificationCenter)
@@ -96,8 +97,8 @@ GS_EXPORT NSString* const GSNetworkNotificationCenterType;
   @private id _internal GS_UNUSED_IVAR;
 #endif
 }
-+ (NSNotificationCenter*) defaultCenter;
-+ (NSNotificationCenter*) notificationCenterForType: (NSString*)type;
++ (id) defaultCenter;
++ (NSDistributedNotificationCenter*) notificationCenterForType: (NSString*)type;
 
 - (void) addObserver: (id)anObserver
 	    selector: (SEL)aSelector
