@@ -14,12 +14,12 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
    */ 
 
 #ifndef __NSNetServices_h_GNUSTEP_BASE_INCLUDE
@@ -93,7 +93,10 @@ enum
 typedef NSUInteger NSNetServicesError;
 
 enum {
-  NSNetServiceNoAutoRename = 1 << 0
+  NSNetServiceNoAutoRename          = 1 << 0
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_9,GS_API_LATEST)
+  ,NSNetServiceListenForConnections = 1 << 1
+#endif
 };
 typedef NSUInteger NSNetServiceOptions;
 
@@ -111,7 +114,7 @@ GS_EXPORT NSString * const NSNetServicesErrorDomain;
 
 
 @protocol  NSNetServiceDelegate
-#ifdef __clang__
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6,GS_API_LATEST) && GS_PROTOCOLS_HAVE_OPTIONAL
 @optional
 #else
 @end
@@ -202,6 +205,22 @@ GS_EXPORT NSString * const NSNetServicesErrorDomain;
 - (void)      netService: (NSNetService *) sender
   didUpdateTXTRecordData: (NSData *) data;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_9,GS_API_LATEST)
+
+/**
+ * Notifies the delegate that the service, which must have been published with
+ * option NSNetServiceListenForConnections, received a new connection.
+ * In order to communicate with the connecting client, you must -open
+ * the streams and schedule them with a runloop.
+ * To reject a connection, just -open and immediately -close both streams.
+ */
+
+- (void)                  netService: (NSNetService *) sender
+  didAcceptConnectionWithInputStream: (NSInputStream *) inputStream
+						outputStream: (NSOutputStream *)outputStream;
+
+#endif
+
 @end
 
 /**
@@ -224,7 +243,7 @@ GS_EXPORT NSString * const NSNetServicesErrorDomain;
  */
 
 @protocol NSNetServiceBrowserDelegate
-#ifdef __clang__
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6,GS_API_LATEST) && GS_PROTOCOLS_HAVE_OPTIONAL
 @optional
 #else
 @end
@@ -336,7 +355,7 @@ GS_EXPORT NSString * const NSNetServicesErrorDomain;
  *   superclass for concrete implementations of its functionality.
  * </p>
  */
-
+GS_EXPORT_CLASS
 @interface NSNetService : NSObject
 {
 #if	GS_EXPOSE(NSNetService)
@@ -362,7 +381,7 @@ GS_EXPORT NSString * const NSNetServicesErrorDomain;
 - (void) scheduleInRunLoop: (NSRunLoop *) aRunLoop
                    forMode: (NSString *) mode;
 
-#if OS_API_VERSION(100500,GS_API_LATEST) 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5,GS_API_LATEST) 
 - (NSInteger) port;
 
 - (void) publishWithOptions: (NSNetServiceOptions)options;
@@ -422,7 +441,7 @@ GS_EXPORT NSString * const NSNetServicesErrorDomain;
  *   functionality.
  * </p>
  */
-
+GS_EXPORT_CLASS
 @interface NSNetServiceBrowser : NSObject
 {
 #if	GS_EXPOSE(NSNetServiceBrowser)
