@@ -4,37 +4,42 @@
    Written by:  Gregory Casamento <greg.casamento@gmail.com>
    Written by:  Richard Frith-Macdonald <rfm@gnu.org>
    Date: 2009,2010
-   
+
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-   
+   Lesser General Public License for more details.
+
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
 
-   */ 
+   */
 
 #ifndef __NSOperation_h_GNUSTEP_BASE_INCLUDE
 #define __NSOperation_h_GNUSTEP_BASE_INCLUDE
 
 #import <Foundation/NSObject.h>
-
-#if OS_API_VERSION(100500, GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
 
 #if	defined(__cplusplus)
 extern "C" {
 #endif
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
+#import <GNUstepBase/GSBlocks.h>
+DEFINE_BLOCK_TYPE_NO_ARGS(GSOperationCompletionBlock, void);
+DEFINE_BLOCK_TYPE_NO_ARGS(GSBlockOperationBlock, void);
+#endif  
+
 @class NSMutableArray;
 
 enum {
@@ -47,6 +52,7 @@ enum {
 
 typedef NSInteger NSOperationQueuePriority;
 
+GS_EXPORT_CLASS
 @interface NSOperation : NSObject
 {
 #if	GS_NONFRAGILE
@@ -78,10 +84,17 @@ typedef NSInteger NSOperationQueuePriority;
  */
 - (void) cancel;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
+/**
+ * Returns the block that will be executed after the operation finishes.
+ */
+- (GSOperationCompletionBlock) completionBlock;
+#endif
+
 /** Returns all the dependencies of the receiver in the order in which they
  * were added.
  */
-- (NSArray *)dependencies;
+- (NSArray *) dependencies;
 
 /** This method should return YES if the -cancel method has been called.<br />
  * NB. a cancelled operation may still be executing.
@@ -136,13 +149,20 @@ typedef NSInteger NSOperationQueuePriority;
  */
 - (void) removeDependency: (NSOperation *)op;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
+/**
+ * Sets the block that will be executed when the operation has finished.
+ */
+- (void) setCompletionBlock: (GSOperationCompletionBlock)aBlock;
+#endif
+
 /** Sets the priority for the receiver.  If the value supplied is not one of
  * the predefined queue priorities, it is converted into the next available
  * defined value moving towards NSOperationQueuePriorityNormal.
  */
 - (void) setQueuePriority: (NSOperationQueuePriority)priority;
 
-#if OS_API_VERSION(100600, GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
 /** Sets the thread priority to be used while executing then -main method.
  * The priority change is implemented in the -start method, so if you are
  * replacing -start you are responsible for managing this.<br />
@@ -166,7 +186,7 @@ typedef NSInteger NSOperationQueuePriority;
  */
 - (void) start;
 
-#if OS_API_VERSION(100600, GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
 /** Returns the thread priority to be used executing the -main method.
  * The default is 0.5
  */
@@ -181,6 +201,20 @@ typedef NSInteger NSOperationQueuePriority;
 
 @end
 
+GS_EXPORT_CLASS
+@interface NSBlockOperation : NSOperation
+{
+  @private
+    NSMutableArray *_executionBlocks;
+    void *_reserved;
+}
+
+// Managing the blocks in the Operation
++ (instancetype) blockOperationWithBlock: (GSBlockOperationBlock)block;
+- (void) addExecutionBlock: (GSBlockOperationBlock)block;
+- (NSArray *) executionBlocks;
+
+@end
 
 /**
  * NSOperationQueue
@@ -191,6 +225,7 @@ enum {
    NSOperationQueueDefaultMaxConcurrentOperationCount = -1
 };
 
+GS_EXPORT_CLASS
 @interface NSOperationQueue : NSObject
 {
 #if	GS_NONFRAGILE
@@ -201,7 +236,7 @@ enum {
 @private id _internal;
 #endif
 }
-#if OS_API_VERSION(100600, GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
 /** If called from within the -main method of an operation which is
  * currently being executed by a queue, this returns the queue instance
  * in use.
@@ -217,12 +252,16 @@ enum {
  */
 - (void) addOperation: (NSOperation *)op;
 
-#if OS_API_VERSION(100600, GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
 /** Adds multiple operations to the receiver and (optionally) waits for
  * all the operations in the queue to finish.
  */
 - (void) addOperations: (NSArray *)ops
      waitUntilFinished: (BOOL)shouldWait;
+  
+/** This method wraps a block in an operation and adds it to the queue.
+ */
+- (void) addOperationWithBlock: (GSBlockOperationBlock)block;
 #endif
 
 /** Cancels all outstanding operations in the queue.
@@ -239,7 +278,7 @@ enum {
  */
 - (NSInteger) maxConcurrentOperationCount;
 
-#if OS_API_VERSION(100600, GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
 /** Return the name of this operation queue.
  */
 - (NSString*) name;
@@ -260,7 +299,7 @@ enum {
  */
 - (void) setMaxConcurrentOperationCount: (NSInteger)cnt;
 
-#if OS_API_VERSION(100600, GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
 /** Sets the name for this operation queue.
  */
 - (void) setName: (NSString*)s;
@@ -277,6 +316,8 @@ enum {
 - (void) waitUntilAllOperationsAreFinished;
 @end
 
+
+
 #if	defined(__cplusplus)
 }
 #endif
