@@ -14,22 +14,23 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
    */ 
 
 #ifndef __NSSortDescriptor_h_GNUSTEP_BASE_INCLUDE
 #define __NSSortDescriptor_h_GNUSTEP_BASE_INCLUDE
 #import	<GNUstepBase/GSVersionMacros.h>
 
-#if OS_API_VERSION(100300,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_3,GS_API_LATEST)
 
 #import	<Foundation/NSObject.h>
 #import	<Foundation/NSArray.h>
+#import	<Foundation/NSSet.h>
 
 #if	defined(__cplusplus)
 extern "C" {
@@ -42,11 +43,25 @@ extern "C" {
  * arrays containging collections or other objects whose properties
  * can be obtained using key names.
  */
+GS_EXPORT_CLASS
 @interface NSSortDescriptor : NSObject <NSCopying, NSCoding>
 {
+#if	GS_EXPOSE(NSSortDescriptor)
+@private
   NSString *_key;
   BOOL	_ascending;
   SEL	_selector;
+  NSComparator _comparator;
+#endif
+#if     GS_NONFRAGILE
+#else
+  /* Pointer to private additional data used to avoid breaking ABI
+   * when we don't have the non-fragile ABI available.
+   * Use this mechanism rather than changing the instance variable
+   * layout (see Source/GSInternal.h for details).
+   */
+  @private id _internal GS_UNUSED_IVAR;
+#endif
 }
 
 /** Returns a flag indicating whether the sort descriptor sorts objects
@@ -62,6 +77,43 @@ extern "C" {
  */
 - (NSComparisonResult) compareObject: (id)object1 toObject: (id)object2;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST) 
+/** <p>Returns an autoreleased sort descriptor for comparisons using the 
+ * 'compare:' selector and the specified key and ordering.</p>
+ *
+ * <p>See also -initWithKey:ascending:.</p>
+ */
++ (id) sortDescriptorWithKey: (NSString *)aKey ascending: (BOOL)ascending;
+
+/** <p>Returns an autoreleased sort descriptor initialized to perform 
+ * comparisons in the specified order using aSelector to compare the property 
+ * aKey of each object.</p>
+ *
+ * <p>See also -initWithKey:ascending:selector:.</p>
+ */
++ (id) sortDescriptorWithKey: (NSString *)aKey 
+                   ascending: (BOOL)ascending 
+                    selector: (SEL)aSelector;
+
+/** <p>Returns an autoreleased sort descriptor initialized to perform 
+ * comparisons in the specified order using the comparator to compare 
+ * the property aKey of each object.</p>
+ *
+ * <p>See also -initWithKey:ascending:selector:.</p>
+ */
++ (id)sortDescriptorWithKey: (NSString *)key 
+                  ascending: (BOOL)ascending 
+                 comparator: (NSComparator)cmptr;
+
+/** <init />
+ * Initialises the receiver to perform comparisons in the specified order
+ * using the comparator to compare the property key of each object.
+ */
+- (id) initWithKey: (NSString *)key
+         ascending: (BOOL)ascending
+        comparator: (NSComparator)cmptr;
+#endif
+
 /** Initialises the receiver for comparisons using the 'compare:' selector
  * and the specified key and ordering.
  */
@@ -70,12 +122,13 @@ extern "C" {
 
 /** <init />
  * Initialises the receiver to perform comparisons in the specified order
- * using selector to compar the property key of each object.
+ * using selector to compare the property key of each object.
  */
 - (id) initWithKey: (NSString *)key
          ascending: (BOOL)ascending
           selector: (SEL)selector;
 
+
 /** Returns the key used to obtain the property on which comparisons are based.
  */
 - (NSString *) key;
@@ -90,13 +143,13 @@ extern "C" {
 - (id) reversedSortDescriptor;
 @end
 
-@interface NSArray (NSSortDescriptorSorting)
+@interface GS_GENERIC_CLASS(NSArray, ElementT) (NSSortDescriptorSorting)
 
 /**
  * Produces a sorted array using the mechanism described for
  * [NSMutableArray-sortUsingDescriptors:]
  */
-- (NSArray *) sortedArrayUsingDescriptors: (NSArray *)sortDescriptors;
+- (GS_GENERIC_CLASS(NSArray, ElementT) *) sortedArrayUsingDescriptors: (NSArray *)sortDescriptors;
 
 @end
 
@@ -117,6 +170,16 @@ extern "C" {
 
 @end
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6,GS_API_LATEST) 
+@interface GS_GENERIC_CLASS(NSSet, ElementT) (NSSortDescriptorSorting)
+ /**
+ * Produces a sorted array from using the mechanism described for
+ * [NSMutableArray-sortUsingDescriptors:]
+ */
+- (GS_GENERIC_CLASS(NSArray, ElementT) *) sortedArrayUsingDescriptors: (NSArray *)sortDescriptors;
+@end
+#endif
+
 #if	defined(__cplusplus)
 }
 #endif
