@@ -11,12 +11,12 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
   */
 
 #ifndef __NSTimeZone_h_GNUSTEP_BASE_INCLUDE
@@ -46,21 +46,22 @@ typedef NSInteger NSTimeZoneNameStyle;
 GS_EXPORT NSString * const NSSystemTimeZoneDidChangeNotification;
 #endif
 
+GS_EXPORT_CLASS
 @interface NSTimeZone : NSObject
 
-//Creating and Initializing an NSTimeZone
+// Creating and Initializing an NSTimeZone
 + (NSTimeZone*) localTimeZone;
 + (NSTimeZone*) timeZoneForSecondsFromGMT: (NSInteger)seconds;
 + (NSTimeZone*) timeZoneWithName: (NSString*)aTimeZoneName;
 
-//Managing Time Zones
+// Managing Time Zones
 + (void) setDefaultTimeZone: (NSTimeZone*)aTimeZone;
 
 // Getting Time Zone Information
 + (NSDictionary*) abbreviationDictionary;
 + (NSArray*) knownTimeZoneNames;
 
-//Getting Arrays of Time Zones
+// Getting Arrays of Time Zones
 + (NSArray*) timeZoneArray;
 - (NSArray*) timeZoneDetailArray;
 
@@ -88,16 +89,16 @@ GS_EXPORT NSString * const NSSystemTimeZoneDidChangeNotification;
 #endif
 
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_5,GS_API_LATEST) 
-- (NSTimeInterval) daylightSavingTimeOffsetForDate: (NSDate *)aDate;
-/** Not implemented */
-- (NSDate *) nextDaylightSavingTimeTransitionAfterDate: (NSDate *)aDate;
+- (NSTimeInterval) daylightSavingTimeOffsetForDate: (NSDate*)aDate;
+
+- (NSDate*) nextDaylightSavingTimeTransitionAfterDate: (NSDate*)aDate;
 
 - (NSTimeInterval) daylightSavingTimeOffset;
-/** Not implemented */
-- (NSDate *) nextDaylightSavingTimeTransition;
 
-- (NSString *)localizedName: (NSTimeZoneNameStyle)style
-                     locale: (NSLocale *)locale;
+- (NSDate*) nextDaylightSavingTimeTransition;
+
+- (NSString*) localizedName: (NSTimeZoneNameStyle)style
+                     locale: (NSLocale*)locale;
 #endif
 
 #if	OS_API_VERSION(GS_API_OPENSTEP, GS_API_MACOSX)
@@ -116,6 +117,7 @@ GS_EXPORT NSString * const NSSystemTimeZoneDidChangeNotification;
 @end
 
 #if	OS_API_VERSION(GS_API_OPENSTEP, GS_API_MACOSX)
+GS_EXPORT_CLASS
 @interface NSTimeZoneDetail : NSTimeZone
 - (BOOL) isDaylightSavingTimeZone;
 - (NSString*) timeZoneAbbreviation;
