@@ -14,19 +14,19 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
    */ 
 
 #ifndef __NSURLRequest_h_GNUSTEP_BASE_INCLUDE
 #define __NSURLRequest_h_GNUSTEP_BASE_INCLUDE
 #import	<GNUstepBase/GSVersionMacros.h>
 
-#if OS_API_VERSION(100200,GS_API_LATEST) && GS_API_VERSION(011300,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_2,GS_API_LATEST) && GS_API_VERSION( 11300,GS_API_LATEST)
 
 #import	<Foundation/NSObject.h>
 
@@ -41,12 +41,17 @@ extern "C" {
 @class NSString;
 @class NSURL;
 
-enum
-{
-  NSURLRequestUseProtocolCachePolicy,
-  NSURLRequestReloadIgnoringCacheData,
-  NSURLRequestReturnCacheDataElseLoad,
-  NSURLRequestReturnCacheDataDontLoad,
+enum {
+    NSURLRequestUseProtocolCachePolicy = 0,
+
+    NSURLRequestReloadIgnoringLocalCacheData = 1,
+    NSURLRequestReloadIgnoringLocalAndRemoteCacheData = 4,
+    NSURLRequestReloadIgnoringCacheData = NSURLRequestReloadIgnoringLocalCacheData,
+
+    NSURLRequestReturnCacheDataElseLoad = 2,
+    NSURLRequestReturnCacheDataDontLoad = 3,
+
+    NSURLRequestReloadRevalidatingCacheData = 5
 };
 /**
  * <deflist>
@@ -76,16 +81,61 @@ enum
  */
 typedef NSUInteger NSURLRequestCachePolicy;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_7,GS_API_LATEST)
+enum
+{
+    NSURLNetworkServiceTypeDefault    = 0,  // Standard internet traffic
+    NSURLNetworkServiceTypeVoIP       = 1,  // Voice over IP control traffic
+    NSURLNetworkServiceTypeVideo      = 2,  // Video traffic
+    NSURLNetworkServiceTypeBackground = 3,  // Background traffic
+    NSURLNetworkServiceTypeVoice      = 4,  // Voice data
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_12,GS_API_LATEST)
+    NSURLNetworkServiceTypeCallSignaling = 11    // Call Signaling - enumeration cases
+#endif
+};
+/**
+ * <deflist>
+ *   <term>NSURLNetworkServiceTypeDefault</term>
+ *   <desc>
+ *     Specifies standard network traffic. Most connections should be made using this service type
+ *     this is the default.
+ *   </desc>
+ *   <term>NSURLNetworkServiceTypeVoIP</term>
+ *   <desc>
+ *     Specifies that the request is for VoIP traffic.
+ *   </desc>
+ *   <term>NSURLNetworkServiceTypeVideo</term>
+ *   <desc>
+ *     Specifies that the request is for video traffic.
+ *   </desc>
+ *   <term>NSURLNetworkServiceTypeBackground</term>
+ *   <desc>
+ *     Specifies that the request is for background traffic.
+ *   </desc>
+ *   <term>NSURLNetworkServiceTypeVoice</term>
+ *   <desc>
+ *     Specifies that the request is for voice traffic.
+ *   </desc>
+ *   <term>NSURLNetworkServiceTypeCallSignaling</term>
+ *   <desc>
+ *     Call Signaling - enumeration cases.
+ *   </desc>
+ * </deflist>
+ */
+typedef NSUInteger NSURLRequestNetworkServiceType;
+#endif
 
 /**
  * This class encapsulates information about a request to load a
  * URL, how to cache the results, and when to deal with a slow/hung
  * load process by timing out.
  */
+GS_EXPORT_CLASS
 @interface NSURLRequest : NSObject <NSCoding, NSCopying, NSMutableCopying>
 {
-@private
+#if	GS_EXPOSE(NSURLRequest)
   void *_NSURLRequestInternal;
+#endif
 }
 
 /*
@@ -93,15 +143,15 @@ typedef NSUInteger NSURLRequestCachePolicy;
  * and with the default cache policy (NSURLRequestUseProtocolCachePolicy)
  * and a sixty second timeout.
  */
-+ (id) requestWithURL: (NSURL *)URL;
++ (instancetype) requestWithURL: (NSURL *)URL;
 
 /**
  * Returns an autoreleased instance initialised with the specified URL,
  * cachePolicy, and timeoutInterval.
  */
-+ (id) requestWithURL: (NSURL *)URL
-	  cachePolicy: (NSURLRequestCachePolicy)cachePolicy
-      timeoutInterval: (NSTimeInterval)timeoutInterval;
++ (instancetype) requestWithURL: (NSURL *)URL
+                    cachePolicy: (NSURLRequestCachePolicy)cachePolicy
+                timeoutInterval: (NSTimeInterval)timeoutInterval;
 
 /**
  * Returns the cache policy associated with the receiver.
@@ -113,15 +163,15 @@ typedef NSUInteger NSURLRequestCachePolicy;
  * and with the default cache policy (NSURLRequestUseProtocolCachePolicy)
  * and a sixty second timeout.
  */
-- (id) initWithURL: (NSURL *)URL;
+- (instancetype) initWithURL: (NSURL *)URL;
 
 /**
  * Initialises the receiver with the specified URL,
  * cachePolicy, and timeoutInterval.
  */
-- (id) initWithURL: (NSURL *)URL
-       cachePolicy: (NSURLRequestCachePolicy)cachePolicy
-   timeoutInterval: (NSTimeInterval)timeoutInterval;
+- (instancetype) initWithURL: (NSURL *)URL
+                 cachePolicy: (NSURLRequestCachePolicy)cachePolicy
+             timeoutInterval: (NSTimeInterval)timeoutInterval;
 
 /**
  * Returns the main document URL for the receiver.<br />
@@ -149,6 +199,7 @@ typedef NSUInteger NSURLRequestCachePolicy;
 
 /**
  */
+GS_EXPORT_CLASS
 @interface NSMutableURLRequest : NSURLRequest
 
 /**
@@ -192,7 +243,7 @@ typedef NSUInteger NSURLRequestCachePolicy;
  */
 - (NSData *) HTTPBody;
 
-#if OS_API_VERSION(100400,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4,GS_API_LATEST)
 /**
  * Returns the currently set stream (if any) to be used to provide data
  * to send as the request body.<br />
@@ -241,7 +292,7 @@ typedef NSUInteger NSURLRequestCachePolicy;
  */
 - (void) setAllHTTPHeaderFields: (NSDictionary *)headerFields;
 
-#if OS_API_VERSION(100400,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4,GS_API_LATEST)
 /**
  * Sets the request body to be the contents of the given stream.<br />
  * The stream should be unopened when it is set, and the load process
@@ -277,6 +328,25 @@ typedef NSUInteger NSURLRequestCachePolicy;
 
 @end
 
+@protocol GSLogDelegate;
+@interface NSMutableURLRequest (GNUstep)
+
+/** Sets a flag to turn on low level debug logging for this request and the
+ * corresponding response.  The previous vaue of the setting is returned.
+ */
+- (int) setDebug: (int)d;
+
+/** Sets a delegate object to override logging of low level I/O of the
+ * request as it is sent and the corresponding response as it arrives.<br />
+ * The delegate object is not retained, so it is the responsibility of the
+ * caller to ensure that it persists until all I/O has completed.<br />
+ * This has no effect unless debug is turned on, but if debug is turned on
+ * it permits the delegate to override the default behavior of writing the
+ * data to stderr.
+ */
+- (id<GSLogDelegate>) setDebugLogDelegate: (id<GSLogDelegate>)d;
+@end
+
 #if	defined(__cplusplus)
 }
 #endif
