@@ -35,6 +35,7 @@
 #import "Foundation/NSDictionary.h"
 #import "Foundation/NSError.h"
 #import "Foundation/NSException.h"
+#import "Foundation/NSMapTable.h"
 #import "Foundation/NSPathUtilities.h"
 #endif
 
@@ -55,6 +56,8 @@
 #endif
 #if     defined(HAVE_UNICODE_UCNV_H)
 #include <unicode/ucnv.h>
+#elif   defined(HAVE_ICU_H)
+#include <icu.h>
 #endif
 
 
@@ -137,7 +140,7 @@ internal_unicode_enc(void)
 #define UNICODE_UTF32 ""
 #endif
 
-static pthread_mutex_t local_lock = PTHREAD_MUTEX_INITIALIZER;
+static gs_mutex_t local_lock = GS_MUTEX_INIT_STATIC;
 
 typedef	unsigned char	unc;
 static NSStringEncoding	defEnc = GSUndefinedEncoding;
@@ -218,7 +221,8 @@ static struct _strenc_ str_encoding_table[] = {
     "NSProprietaryStringEncoding","",0,0,0},
 #endif
 
-// GNUstep additions
+  /* GNUstep additions
+   */
   {NSISOCyrillicStringEncoding,
     "NSISOCyrillicStringEncoding","ISO-8859-5",0,1,0},
   {NSKOI8RStringEncoding,
@@ -247,15 +251,77 @@ static struct _strenc_ str_encoding_table[] = {
     "NSISOLatin9StringEncoding","ISO-8859-15",1,1,0},
   {NSUTF7StringEncoding,
     "NSUTF7StringEncoding","UTF-7",0,0,0},
-  {NSGB2312StringEncoding,
-    "NSGB2312StringEncoding","EUC-CN",0,0,0},
+  {NSChineseEUCStringEncoding,
+    "NSChineseEUCStringEncoding","EUC-CN",0,0,0},
   {NSGSM0338StringEncoding,
     "NSGSM0338StringEncoding","",0,1,0},
-  {NSBIG5StringEncoding,
-    "NSBIG5StringEncoding","BIG5",0,0,0},
+  {NSBig5StringEncoding,
+    "NSBig5StringEncoding","BIG5",0,0,0},
   {NSKoreanEUCStringEncoding,
     "NSKoreanEUCStringEncoding","EUC-KR",0,0,0},
 
+  /* DOS and Windows encodings
+   */
+  {NSDOSLatinUSStringEncoding,
+    "NSDOSLatinUSStringEncoding","CP437",0,0,0},
+  {NSDOSGreekStringEncoding,
+    "NSDOSGreekStringEncoding","CP737",0,0,0},
+  {NSDOSBalticRimStringEncoding,
+    "NSDOSBalticRimStringEncoding","CP775",0,0,0},
+  {NSDOSLatin1StringEncoding,
+    "NSDOSLatin1StringEncoding","CP850",0,0,0},
+  {NSDOSGreek1StringEncoding,
+    "NSDOSGreek1StringEncoding","CP851",0,0,0},
+  {NSDOSLatin2StringEncoding,
+    "NSDOSLatin2StringEncoding","CP852",0,0,0},
+  {NSDOSCyrillicStringEncoding,
+    "NSDOSCyrillicStringEncoding","CP855",0,0,0},
+  {NSDOSTurkishStringEncoding,
+    "NSDOSTurkishStringEncoding","CP857",0,0,0},
+  {NSDOICortugueseStringEncoding,
+    "NSDOICortugueseStringEncoding","CP860",0,0,0},
+  {NSDOSIcelandicStringEncoding,
+    "NSDOSIcelandicStringEncoding","CP861",0,0,0},
+  {NSDOSHebrewStringEncoding,
+    "NSDOSHebrewStringEncoding","CP862",0,0,0},
+  {NSDOSCanadianFrenchStringEncoding,
+    "NSDOSCanadianFrenchStringEncoding","CP863",0,0,0},
+  {NSDOSArabicStringEncoding,
+    "NSDOSArabicStringEncoding","CP864",0,0,0},
+  {NSDOSNordicStringEncoding,
+    "NSDOSNordicStringEncoding","CP865",0,0,0},
+  {NSDOSRussianStringEncoding,
+    "NSDOSRussianStringEncoding","CP866",0,0,0},
+  {NSDOSGreek2StringEncoding,
+    "NSDOSGreek2StringEncoding","CP869",0,0,0},
+  {NSDOSThaiStringEncoding,
+    "NSDOSThaiStringEncoding","CP874",0,0,0},
+  {NSDOSJapaneseStringEncoding,
+    "NSDOSJapaneseStringEncoding","CP932",0,0,0},
+  {NSDOSChineseSimplifStringEncoding,
+    "NSDOSChineseSimplifStringEncoding","CP936",0,0,0},
+  {NSDOSKoreanStringEncoding,
+    "NSDOSKoreanStringEncoding","CP949",0,0,0},
+  {NSDOSChineseTradStringEncoding,
+    "NSDOSChineseTradStringEncoding","CP950",0,0,0},
+  {NSWindowsHebrewStringEncoding,
+    "NSWindowsHebrewStringEncoding","CP1255",0,0,0},
+  {NSWindowsArabicStringEncoding,
+    "NSWindowsArabicStringEncoding","CP1256",0,0,0},
+  {NSWindowsBalticRimStringEncoding,
+    "NSWindowsBalticRimStringEncoding","CP1257",0,0,0},
+  {NSWindowsVietnameseStringEncoding,
+    "NSWindowsVietnameseStringEncoding","CP1258",0,0,0},
+  {NSWindowsKoreanJohabStringEncoding,
+    "NSWindowsKoreanJohabStringEncoding","CP1361",0,0,0},
+
+  {NSGB_2312_80StringEncoding,	// Same as NSChineseEUCStringEncoding
+    "NSGB_2312_80StringEncoding","EUC-CN",0,0,0},
+  {NSGBK_95StringEncoding,	// Same as NSDOSChineseSimplifStringEncoding
+    "NSGBK_95StringEncoding","CP936",0,0,0},
+  {NSGB_18030_2000StringEncoding,
+    "NSGB_18030_2000StringEncoding","GB18030",0,0,0},
+
 /* Now Apple encodings which have high numeric values.
  */
   {NSUTF16BigEndianStringEncoding,
@@ -272,74 +338,50 @@ static struct _strenc_ str_encoding_table[] = {
   {0,"Unknown encoding","",0,0,0}
 };
 
-static struct _strenc_	**encodingTable = 0;
 static unsigned		encTableSize = 0;
+static NSMapTable   	*encodingPointerTable = nil;
 
 static void GSSetupEncodingTable(void)
 {
-  if (encodingTable == 0)
+  if (nil == encodingPointerTable)
     {
-      (void)pthread_mutex_lock(&local_lock);
-      if (encodingTable == 0)
+      GS_MUTEX_LOCK(local_lock);
+      if (nil == encodingPointerTable)
 	{
-	  static struct _strenc_	**encTable = 0;
-	  unsigned			count;
 	  unsigned			i;
 
 	  /*
 	   * We want to store pointers to our string encoding info in a
 	   * large table so we can do efficient lookup by encoding value.
 	   */
-#define	MAX_ENCODING	128
-	  count = sizeof(str_encoding_table) / sizeof(struct _strenc_);
-
-	  /*
-	   * First determine the largest encoding value and create a
-	   * large enough table of pointers.
-	   */
-	  encTableSize = 0;
-	  for (i = 0; i < count; i++)
-	    {
-	      unsigned	tmp = str_encoding_table[i].enc;
-
-	      if (tmp > encTableSize)
-		{
-		  if (tmp < MAX_ENCODING)
-		    {
-		      encTableSize = tmp;
-		    }
-		}
-	    }
-	  encTable = malloc(
-	    (encTableSize+1)*sizeof(struct _strenc_ *));
-	  memset(encTable, 0, (encTableSize+1)*sizeof(struct _strenc_ *));
+	  encTableSize = sizeof(str_encoding_table) / sizeof(struct _strenc_);
+	  encodingPointerTable = NSCreateMapTable(NSIntegerMapKeyCallBacks,
+	    NSNonOwnedPointerMapValueCallBacks, encTableSize);
+	  RELEASE([NSObject leakAt:&encodingPointerTable]);
 
 	  /*
 	   * Now set up the pointers at the correct location in the table.
 	   */
-	  for (i = 0; i < count; i++)
+	  for (i = 0; i < encTableSize; i++)
 	    {
 	      struct _strenc_ *entry = &str_encoding_table[i];
-	      unsigned	tmp = entry->enc;
 
-	      if (tmp < MAX_ENCODING)
-		{
-		  encTable[tmp] = entry;
-		}
+	      NSMapInsert(encodingPointerTable,
+		(const void *)entry->enc, (const void *)entry);
 #ifdef HAVE_ICONV
 	      if (entry->iconv != 0 && *(entry->iconv) != 0)
 		{
 		  iconv_t	c;
 		  int		l;
-		  char	*lossy;
+		  char		*lossy;
 
 		  /*
 		   * See if we can do a lossy conversion.
 		   */
 		  l = strlen(entry->iconv);
 		  lossy = malloc(l + 11);
-		  strncpy(lossy, entry->iconv, l);
-		  strncpy(lossy + l, "//TRANSLIT", 11);
+		  memcpy(lossy, entry->iconv, l);
+		  memcpy(lossy + l, "//TRANSLIT", 11);
 		  c = iconv_open(lossy, UNICODE_ENC);
 		  if (c == (iconv_t)-1)
 		    {
@@ -353,9 +395,8 @@ static void GSSetupEncodingTable(void)
 		}
 #endif
 	    }
-	  encodingTable = encTable;
 	}
-      (void)pthread_mutex_unlock(&local_lock);
+      GS_MUTEX_UNLOCK(local_lock);
     }
 }
 
@@ -364,27 +405,10 @@ EntryForEncoding(NSStringEncoding enc)
 {
   struct _strenc_ *entry = 0;
 
-  if (enc > 0)
+  if (enc != 0)
     {
       GSSetupEncodingTable();
-      if (enc <= encTableSize)
-	{
-	  entry = encodingTable[enc];
-	}
-      else
-	{
-	  unsigned	i = 0;
-
-	  while (i < sizeof(str_encoding_table) / sizeof(struct _strenc_))
-	    {
-	      if (str_encoding_table[i].enc == enc)
-		{
-		  entry = &str_encoding_table[i];
-		  break;
-		}
-	      i++;
-	    }
-	}
+      entry = NSMapGet(encodingPointerTable, (const void *)enc);
     }
   return entry;
 }
@@ -460,7 +484,7 @@ GSPrivateIsEncodingSupported(NSStringEncoding enc)
  *  character set registry and encoding information. For instance,
  *  for the iso8859-5 character set, the registry is iso8859 and
  *  the encoding is 5, and the returned NSStringEncoding is
- *  NSISOCyrillicStringEncoding. If there is no specific encoding,
+ *  NSISOLatinCyrillicStringEncoding. If there is no specific encoding,
  *  use @"0". Returns GSUndefinedEncoding if there is no match.
  */
 NSStringEncoding
@@ -1936,7 +1960,7 @@ GSFromUnicode(unsigned char **dst, unsigned int *size, const unichar *src,
 		      u2 = src[spos++];
 		      u2 = (((u2 & 0xff00) >> 8) + ((u2 & 0x00ff) << 8));
 
-		      if ((u2 < 0xdc00) && (u2 > 0xdfff))
+		      if ((u2 < 0xdc00) || (u2 > 0xdfff))
 			{
 			  spos--;
 			  if (strict)
@@ -2054,7 +2078,7 @@ GSFromUnicode(unsigned char **dst, unsigned int *size, const unichar *src,
 		      /* get second unichar */
 		      u2 = src[spos++];
 
-		      if ((u2 < 0xdc00) && (u2 > 0xdfff))
+		      if ((u2 < 0xdc00) || (u2 > 0xdfff))
 			{
 			  spos--;
 			  if (strict)
@@ -2242,12 +2266,22 @@ GSFromUnicode(unsigned char **dst, unsigned int *size, const unichar *src,
                         }
                       else
                         {
-                          dpos += sprintf((char*)&ptr[dpos], "\\%03o", u);
+                          char octchars[] = "01234567";
+                          ptr[dpos++] = '\\';
+                          ptr[dpos++] = octchars[(u >> 6) & 7];
+                          ptr[dpos++] = octchars[(u >> 3) & 7];
+                          ptr[dpos++] = octchars[u & 7];
                         }
                     }
                   else
                     {
-                      dpos += sprintf((char*)&ptr[dpos], "\\u%04x", u);
+                      char hexchars[] = "0123456789abcdef";
+                      ptr[dpos++] = '\\';
+                      ptr[dpos++] = 'u';
+                      ptr[dpos++] = hexchars[(u >> 12) & 0xF];
+                      ptr[dpos++] = hexchars[(u >> 8) & 0xF];
+                      ptr[dpos++] = hexchars[(u >> 4) & 0xF];
+                      ptr[dpos++] = hexchars[u & 0xF];
                     }
                 }
             }
@@ -2725,7 +2759,7 @@ GSPrivateAvailableEncodings()
   if (_availableEncodings == 0)
     {
       GSSetupEncodingTable();
-      (void)pthread_mutex_lock(&local_lock);
+      GS_MUTEX_LOCK(local_lock);
       if (_availableEncodings == 0)
 	{
 	  NSStringEncoding	*encodings;
@@ -2741,17 +2775,19 @@ GSPrivateAvailableEncodings()
 	   */
 	  encodings = malloc(sizeof(NSStringEncoding) * (encTableSize+1));
 	  pos = 0;
-	  for (i = 0; i < encTableSize+1; i++)
+	  for (i = 0; i < encTableSize; i++)
 	    {
-	      if (GSPrivateIsEncodingSupported(i) == YES)
+	      NSStringEncoding encoding = str_encoding_table[i].enc;
+
+	      if (GSPrivateIsEncodingSupported(encoding) == YES)
 		{
-		  encodings[pos++] = i;
+		  encodings[pos++] = encoding;
 		}
 	    }
 	  encodings[pos] = 0;
 	  _availableEncodings = encodings;
 	}
-      (void)pthread_mutex_unlock(&local_lock);
+      GS_MUTEX_UNLOCK(local_lock);
     }
   return _availableEncodings;
 }
@@ -2852,15 +2888,96 @@ GSPrivateCStringEncoding(const char *encoding)
         || strcmp(encoding, "eucCN") == 0 /* IRIX NetBSD */
         || strcmp(encoding, "IBM-eucCN") == 0 /* AIX */
         || strcmp(encoding, "hp15CN") == 0 /* HP-UX */)
-        enc = NSGB2312StringEncoding;
+        enc = NSChineseEUCStringEncoding;
       else if (strcmp(encoding, "BIG5") == 0 /* glibc Solaris NetBSD */
         || strcmp(encoding, "big5") == 0 /* AIX HP-UX OSF/1 */)
-        enc = NSBIG5StringEncoding;
+        enc = NSBig5StringEncoding;
       else if (strcmp(encoding, "EUC-KR") == 0 /* glibc */
         || strcmp(encoding, "eucKR") == 0 /* HP-UX IRIX OSF/1 NetBSD */
         || strcmp(encoding, "IBM-eucKR") == 0 /* AIX */
         || strcmp(encoding, "5601") == 0 /* Solaris */)
         enc = NSKoreanEUCStringEncoding;
+      else if (strcmp(encoding, "CP437") == 0
+        || strcmp(encoding, "IBM-437") == 0)
+        enc = NSDOSLatinUSStringEncoding;
+      else if (strcmp(encoding, "CP737") == 0
+        || strcmp(encoding, "IBM-737") == 0)
+        enc = NSDOSGreekStringEncoding;
+      else if (strcmp(encoding, "CP775") == 0
+        || strcmp(encoding, "IBM-775") == 0)
+        enc = NSDOSBalticRimStringEncoding;
+      else if (strcmp(encoding, "CP850") == 0
+        || strcmp(encoding, "IBM-850") == 0)
+        enc = NSDOSLatin1StringEncoding;
+      else if (strcmp(encoding, "CP851") == 0
+        || strcmp(encoding, "IBM-851") == 0)
+        enc = NSDOSGreek1StringEncoding;
+      else if (strcmp(encoding, "CP852") == 0
+        || strcmp(encoding, "IBM-852") == 0)
+        enc = NSDOSLatin2StringEncoding;
+      else if (strcmp(encoding, "CP855") == 0
+        || strcmp(encoding, "IBM-855") == 0)
+        enc = NSDOSCyrillicStringEncoding;
+      else if (strcmp(encoding, "CP857") == 0
+        || strcmp(encoding, "IBM-857") == 0)
+        enc = NSDOSTurkishStringEncoding;
+      else if (strcmp(encoding, "CP860") == 0
+        || strcmp(encoding, "IBM-860") == 0)
+        enc = NSDOICortugueseStringEncoding;
+      else if (strcmp(encoding, "CP861") == 0
+        || strcmp(encoding, "IBM-861") == 0)
+        enc = NSDOSIcelandicStringEncoding;
+      else if (strcmp(encoding, "CP862") == 0
+        || strcmp(encoding, "IBM-862") == 0)
+        enc = NSDOSHebrewStringEncoding;
+      else if (strcmp(encoding, "CP863") == 0
+        || strcmp(encoding, "IBM-863") == 0)
+        enc = NSDOSCanadianFrenchStringEncoding;
+      else if (strcmp(encoding, "CP864") == 0
+        || strcmp(encoding, "IBM-864") == 0)
+        enc = NSDOSArabicStringEncoding;
+      else if (strcmp(encoding, "CP865") == 0
+        || strcmp(encoding, "IBM-865") == 0)
+        enc = NSDOSNordicStringEncoding;
+      else if (strcmp(encoding, "CP866") == 0
+        || strcmp(encoding, "IBM-866") == 0)
+        enc = NSDOSRussianStringEncoding;
+      else if (strcmp(encoding, "CP869") == 0
+        || strcmp(encoding, "IBM-869") == 0)
+        enc = NSDOSGreek2StringEncoding;
+      else if (strcmp(encoding, "CP874") == 0
+        || strcmp(encoding, "IBM-874") == 0)
+        enc = NSDOSThaiStringEncoding;
+      else if (strcmp(encoding, "CP932") == 0
+        || strcmp(encoding, "IBM-932") == 0)
+        enc = NSDOSJapaneseStringEncoding;
+      else if (strcmp(encoding, "CP936") == 0
+        || strcmp(encoding, "IBM-936") == 0
+        || strcmp(encoding, "GBK") == 0)
+        enc = NSDOSChineseSimplifStringEncoding;
+      else if (strcmp(encoding, "CP949") == 0
+        || strcmp(encoding, "IBM-949") == 0)
+        enc = NSDOSKoreanStringEncoding;
+      else if (strcmp(encoding, "CP950") == 0
+        || strcmp(encoding, "IBM-950") == 0)
+        enc = NSDOSChineseTradStringEncoding;
+      else if (strcmp(encoding, "CP1255") == 0
+        || strcmp(encoding, "WINDOWS-1255") == 0)
+        enc = NSWindowsHebrewStringEncoding;
+      else if (strcmp(encoding, "CP1256") == 0
+        || strcmp(encoding, "WINDOWS-1256") == 0)
+        enc = NSWindowsArabicStringEncoding;
+      else if (strcmp(encoding, "CP1257") == 0
+        || strcmp(encoding, "WINDOWS-1257") == 0)
+        enc = NSWindowsBalticRimStringEncoding;
+      else if (strcmp(encoding, "CP1258") == 0
+        || strcmp(encoding, "WINDOWS-1258") == 0)
+        enc = NSWindowsVietnameseStringEncoding;
+      else if (strcmp(encoding, "CP1361") == 0
+        || strcmp(encoding, "WINDOWS-1361") == 0)
+        enc = NSWindowsKoreanJohabStringEncoding;
+      else if (strcmp(encoding, "GB18030") == 0)
+        enc = NSGB_18030_2000StringEncoding;
     }
 
   if (enc == GSUndefinedEncoding)
@@ -2893,10 +3010,10 @@ GSPrivateDefaultCStringEncoding()
 
       GSSetupEncodingTable();
 
-      (void)pthread_mutex_lock(&local_lock);
+      GS_MUTEX_LOCK(local_lock);
       if (defEnc != GSUndefinedEncoding)
 	{
-	  (void)pthread_mutex_unlock(&local_lock);
+	  GS_MUTEX_UNLOCK(local_lock);
 	  return defEnc;
 	}
 
@@ -2917,9 +3034,9 @@ GSPrivateDefaultCStringEncoding()
 	  else
 	    {
 	      fprintf(stderr,
-		      "WARNING: %s - encoding not supported.\n", encoding);
+		"WARNING: %s - encoding not supported.\n", encoding);
 	      fprintf(stderr,
-		      "  NSISOLatin1StringEncoding set as default.\n");
+		"  NSISOLatin1StringEncoding set as default.\n");
 	      defEnc = NSISOLatin1StringEncoding;
 	    }
 	}
@@ -2934,13 +3051,13 @@ GSPrivateDefaultCStringEncoding()
       else if (GSPrivateIsEncodingSupported(defEnc) == NO)
 	{
 	  fprintf(stderr, "WARNING: %s - encoding not implemented as "
-		  "default c string encoding.\n", encoding);
+	    "default c string encoding.\n", encoding);
 	  fprintf(stderr,
-		  "  NSISOLatin1StringEncoding set as default.\n");
+	    "  NSISOLatin1StringEncoding set as default.\n");
 	  defEnc = NSISOLatin1StringEncoding;
 	}
 
-      (void)pthread_mutex_unlock(&local_lock);
+      GS_MUTEX_UNLOCK(local_lock);
     }
   return defEnc;
 }
@@ -3013,7 +3130,7 @@ GSPrivateICUCStringEncoding()
   if (icuEnc == GSUndefinedEncoding)
     {
       const char        *encoding = 0;
-#if HAVE_UNICODE_UCNV_H
+#if defined(HAVE_UNICODE_UCNV_H) || defined(HAVE_ICU_H)
       const char *defaultName;
       UErrorCode err = U_ZERO_ERROR;
 
