@@ -15,12 +15,12 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
 
    $Date$ $Revision$
    */
@@ -44,10 +44,6 @@ static SEL	oaiSel;
 
 static Class	GSInlineArrayClass;
 /* This class stores objects inline in data beyond the end of the instance.
- * However, when GC is enabled the object data is typed, and all data after
- * the end of the class is ignored by the garbage collector (which would
- * mean that objects in the array could be collected).
- * We therefore do not provide the class when GC is being used.
  */
 @interface GSInlineArray : GSArray
 {
@@ -73,6 +69,18 @@ static Class	GSInlineArrayClass;
 
 @implementation GSArray
 
+- (NSUInteger) sizeOfContentExcluding: (NSHashTable*)exclude
+{
+  NSUInteger	size = _count * sizeof(id);
+  NSUInteger	index = _count;
+
+  while (index-- > 0)
+    {
+      size += [_contents_array[index] sizeInBytesExcluding: exclude];
+    }
+  return size + [super sizeOfContentExcluding: exclude];
+}
+
 - (void) _raiseRangeExceptionWithIndex: (NSUInteger)index from: (SEL)sel
 {
   NSDictionary *info;
@@ -421,6 +429,13 @@ static Class	GSInlineArrayClass;
 
 @implementation GSMutableArray
 
+- (NSUInteger) sizeOfContentExcluding: (NSHashTable*)exclude
+{
+  /* Can't safely calculate for mutable object; just buffer size
+   */
+  return _capacity * sizeof(void*);
+}
+
 + (void) initialize
 {
   if (self == [GSMutableArray class])
@@ -938,22 +953,6 @@ static Class	GSInlineArrayClass;
   return count;
 }
 
-- (NSUInteger) sizeInBytesExcluding: (NSHashTable*)exclude
-{
-  NSUInteger	size = GSPrivateMemorySize(self, exclude);
-
-  if (size > 0)
-    {
-      NSUInteger	count = _count;
-
-      size += _capacity*sizeof(void*);
-      while (count-- > 0)
-	{
-	  size += [_contents_array[count] sizeInBytesExcluding: exclude];
-	}
-    }
-  return size;
-}
 @end
 
 
@@ -965,7 +964,7 @@ static Class	GSInlineArrayClass;
   if ((self = [super init]) != nil)
     {
       array = anArray;
-      IF_NO_GC(RETAIN(array));
+      IF_NO_ARC(RETAIN(array);)
       pos = 0;
     }
   return self;
