@@ -60,7 +60,7 @@
 {
 @public
   GSIMapTable_t	map;
-  NSUInteger _version;
+  unsigned long	_version;
 }
 @end
 
@@ -213,7 +213,7 @@ static SEL	objSel;
       node = GSIMapNodeForKey(&map, (GSIMapKey)(id)keys[i]);
       if (node)
 	{
-	  IF_NO_GC(RETAIN(objs[i]));
+	  IF_NO_ARC(RETAIN(objs[i]);)
 	  RELEASE(node->value.obj);
 	  node->value.obj = objs[i];
 	}
@@ -426,7 +426,6 @@ static SEL	objSel;
 {
   GSIMapNode	node;
 
-  _version++;
   if (aKey == nil)
     {
       NSException	*e;
@@ -448,10 +447,11 @@ static SEL	objSel;
 				userInfo: self];
       [e raise];
     }
+  _version++;
   node = GSIMapNodeForKey(&map, (GSIMapKey)aKey);
   if (node)
     {
-      IF_NO_GC(RETAIN(anObject));
+      IF_NO_ARC(RETAIN(anObject);)
       RELEASE(node->value.obj);
       node->value.obj = anObject;
     }
@@ -485,7 +485,7 @@ static SEL	objSel;
 				   objects: (__unsafe_unretained id[])stackbuf
 				     count: (NSUInteger)len
 {
-  state->mutationsPtr = (unsigned long *)&_version;
+  state->mutationsPtr = &_version;
   return GSIMapCountByEnumeratingWithStateObjectsCount
     (&map, state, stackbuf, len);
 }
