@@ -2218,7 +2218,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
 - (void) receivedEventWrite
 {
-  NSString	*operation;
+  NSString		*operation;
   NSMutableDictionary	*info;
 
   info = [writeInfo objectAtIndex: 0];
@@ -2257,14 +2257,14 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
     }
   else
     {
-      NSData	*item;
+      NSData		*item;
       int		length;
       const void	*ptr;
 
       item = [info objectForKey: NSFileHandleNotificationDataItem];
       length = [item length];
       ptr = [item bytes];
-      if (writePos < length)
+      while (writePos < length)
         {
           int	written;
 
@@ -2272,7 +2272,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
                          length: length-writePos];
           if (written <= 0)
             {
-	      if (written < 0 && errno != EAGAIN && errno != EINTR)
+	      if (errno != EAGAIN && errno != EINTR)
 	        {
 	          NSString	*s;
 
@@ -2281,6 +2281,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 	          [info setObject: s forKey: GSFileHandleNotificationError];
 	          [self postWriteNotification];
 	        }
+	      break;
 	    }
 	  else
             {
