@@ -113,9 +113,8 @@ static RunLoopEventType typeForStream(NSStream *aStream)
   RunLoopEventType 	type = typeForStream(aStream);
   void			*event = [aStream _loopID];
 
-  NSDebugMLLog(@"NSStream",
-    @"-addStream:mode: %@ (desc %d,%d) to %@ mode %@",
-    aStream, (int)(intptr_t)event, type, self, mode);
+  NSDebugMLLog(@"NSStream", @"%@ (type %d) to %@ mode %@",
+    aStream, type, self, mode);
   [self addEvent: event
 	    type: type
 	 watcher: (id<RunLoopEvents>)aStream
@@ -211,6 +210,7 @@ static RunLoopEventType typeForStream(NSStream *aStream)
     && _currentStatus != NSStreamStatusOpening)
     {
       NSDebugMLLog(@"NSStream", @"Attempt to re-open stream %@", self);
+      return;
     }
   [self _setStatus: NSStreamStatusOpen];
   [self _schedule];
@@ -240,13 +240,17 @@ static RunLoopEventType typeForStream(NSStream *aStream)
       modes = (NSMutableArray*)NSMapGet(_loops, (void*)aRunLoop);
       if ([modes containsObject: mode])
 	{
-	  [aRunLoop removeStream: self mode: mode];
+	  [self _removeFromRunLoop: aRunLoop forMode: mode];
 	  [modes removeObject: mode];
 	  if ([modes count] == 0)
 	    {
 	      NSMapRemove(_loops, (void*)aRunLoop);
 	    }
 	}
+      if (NSCountMapTable(_loops) == 0)
+	{
+	  _scheduled = NO;
+	}
     }
 }
 
@@ -274,7 +278,7 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 	   */
 	  if ([self _isOpened])
 	    {
-	      [aRunLoop addStream: self mode: mode];
+	      [self _scheduleInRunLoop: aRunLoop forMode: mode];
 	    }
 	}
     }
@@ -379,6 +383,11 @@ static RunLoopEventType typeForStream(NSStream *aStream)
   return;
 }
 
+- (void) _removeFromRunLoop: (NSRunLoop *)aRunLoop forMode: (NSString *)mode
+{
+  [aRunLoop removeStream: self mode: mode];
+}
+
 - (void) _resetEvents: (NSUInteger)mask
 {
   return;
@@ -388,6 +397,16 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 {
 }
 
+- (BOOL) _scheduled
+{
+  return NO;
+}
+
+- (void) _scheduleInRunLoop: (NSRunLoop*)loop forMode: (NSString*)mode
+{
+  [loop addStream: self mode: mode];
+}
+
 - (void) _sendEvent: (NSStreamEvent)event
 {
 }
@@ -479,7 +498,7 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 
 - (void) _recordError: (NSError*)anError
 {
-  NSDebugMLLog(@"NSStream", @"record error: %@ - %@", self, anError);
+  NSDebugMLLog(@"NSStream", @"%@ - %@", self, anError);
   ASSIGN(_lastError, anError);
   [self _setStatus: NSStreamStatusError];
 }
@@ -502,12 +521,23 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 
       while (i-- > 0)
 	{
-	  [k addStream: self mode: [v objectAtIndex: i]];
+          [self _scheduleInRunLoop: k forMode: [v objectAtIndex: i]];
 	}
     }
   NSEndMapTableEnumeration(&enumerator);
 }
 
+- (BOOL) _scheduled
+{
+  return _scheduled;
+}
+
+- (void) _scheduleInRunLoop: (NSRunLoop*)loop forMode: (NSString*)mode
+{
+  [loop addStream: self mode: mode];
+  _scheduled = YES;
+}
+
 - (void) _sendEvent: (NSStreamEvent)event
 {
   [self _sendEvent: event delegate: _delegateValid == YES ? _delegate : nil];
@@ -515,6 +545,8 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 
 - (void) _sendEvent: (NSStreamEvent)event delegate: (id)delegate
 {
+  NSDebugMLLog(@"NSStream",
+    @"%@ sendEvent %@", self, [self stringFromEvent:event]);
   if (event == NSStreamEventNone)
     {
       return;
@@ -566,6 +598,10 @@ static RunLoopEventType typeForStream(NSStream *aStream)
       if ((_events & NSStreamEventHasSpaceAvailable) == 0)
 	{
 	  _events |= NSStreamEventHasSpaceAvailable;
+	  if (_currentStatus == NSStreamStatusWriting)
+	    {
+	      [self _setStatus: NSStreamStatusOpen];
+	    }
 	  if (delegate != nil)
 	    {
 	      [delegate stream: self
@@ -657,6 +693,7 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 	}
     }
   NSEndMapTableEnumeration(&enumerator);
+  _scheduled = NO;
 }
 
 - (BOOL) runLoopShouldBlock: (BOOL*)trigger
