@@ -59,9 +59,6 @@
 
 typedef	unsigned char	uchar;
 
-NSString * const NSInconsistentArchiveException =
-  @"NSInconsistentArchiveException";
-
 #define	PREFIX		"GNUstep archive"
 
 static SEL serSel;
@@ -301,7 +298,7 @@ static Class	NSMutableDataMallocClass;
       case _C_ULNG_LNG:	info = _GSC_ULNG_LNG | _GSC_S_LNG_LNG;	break;
       case _C_FLT:	info = _GSC_FLT;	break;
       case _C_DBL:	info = _GSC_DBL;	break;
-#if __GNUC__ > 2 && defined(_C_BOOL)
+#if defined(_C_BOOL) && (!defined(__GNUC__) || __GNUC__ > 2)
       case _C_BOOL:	info = _GSC_BOOL;	break;
 #endif
       default:		info = _GSC_NONE;	break;
@@ -675,7 +672,7 @@ static Class	NSMutableDataMallocClass;
 	(*_serImp)(_dst, serSel, (void*)buf, @encode(double), nil);
 	return;
 
-#if __GNUC__ > 2 && defined(_C_BOOL)
+#if defined(_C_BOOL) && (!defined(__GNUC__) || __GNUC__ > 2)
       case _C_BOOL:
 	(*_tagImp)(_dst, tagSel, _GSC_BOOL);
 	(*_serImp)(_dst, serSel, (void*)buf, @encode(_Bool), nil);
