@@ -57,11 +57,6 @@
 
 #import "GSPrivate.h"
 
-/* Constants */
-NSString * const NSBundleDidLoadNotification = @"NSBundleDidLoadNotification";
-NSString * const NSShowNonLocalizedStrings = @"NSShowNonLocalizedStrings";
-NSString * const NSLoadedClasses = @"NSLoadedClasses";
-
 static NSFileManager	*
 manager()
 {
@@ -393,7 +388,7 @@ GSPrivateExecutablePath()
 	      executablePath = [executablePath stringByResolvingSymlinksInPath];
 	      executablePath = [executablePath stringByStandardizingPath];
 	    }
-	  IF_NO_GC([executablePath retain];)
+	  IF_NO_ARC([executablePath retain];)
 	  beenHere = YES;
 	}
       [load_lock unlock];
@@ -1071,7 +1066,6 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
 {
   if (self == [NSBundle class])
     {
-      extern const char	*GSPathHandling(const char *);
       NSAutoreleasePool *pool = [NSAutoreleasePool new];
       NSString          *file;
       const char	*mode;
@@ -1633,7 +1627,7 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
   if (_byIdentifier)
     {
       bundle = (NSBundle *)NSMapGet(_byIdentifier, identifier);
-IF_NO_GC(
+IF_NO_ARC(
 	[bundle retain]; /* retain - look as if we were alloc'ed */
 )
     }
@@ -1726,7 +1720,7 @@ IF_NO_GC(
   bundle = (NSBundle *)NSMapGet(_bundles, path);
   if (bundle != nil)
     {
-      IF_NO_GC([bundle retain];)
+      IF_NO_ARC([bundle retain];)
       [load_lock unlock];
       [self dealloc];
       return bundle;
@@ -1776,7 +1770,7 @@ IF_NO_GC(
         {
           if (bundle != nil)
             {
-              IF_NO_GC([bundle retain];)
+              IF_NO_ARC([bundle retain];)
               [load_lock unlock];
               [self dealloc];
               return bundle;
@@ -1806,7 +1800,7 @@ IF_NO_GC(
        * dynamically loaded code, so we want to prevent a bundle
        * being loaded twice.
        */
-      IF_NO_GC([self retain];)
+      IF_NO_ARC([self retain];)
       return;
     }
   if (_path != nil)
@@ -2029,7 +2023,7 @@ IF_NO_GC(
 	 We need it to answer calls like bundleForClass:; also, users
 	 normally want all loaded bundles to appear when they call
 	 +allBundles.  */
-      IF_NO_GC([self retain];)
+      IF_NO_ARC([self retain];)
 
       classNames = [NSMutableArray arrayWithCapacity: [_bundleClasses count]];
       classEnumerator = [_bundleClasses objectEnumerator];
@@ -2228,6 +2222,24 @@ IF_NO_GC(
 	}
     }
 
+#ifdef __ANDROID__
+  /* Android: check for directory resources by passing file path as subpath,
+   * as AAssetDir and thereby NSDirectoryEnumerator doesn't list directories
+   */
+  subPath = subPath ? [subPath stringByAppendingPathComponent: file] : file;
+  pathlist = [[self _bundleResourcePathsWithRootPath: rootPath
+    subPath: subPath localization: nil] objectEnumerator];
+  while ((path = [pathlist nextObject]) != nil)
+    {
+      NSString *lastPathComponent = [path lastPathComponent];
+      if ([lastPathComponent isEqualToString:file]
+        && [mgr isReadableFileAtPath: path])
+        {
+          return path;
+        }
+    }
+#endif /* __ANDROID__ */
+
   return nil;
 }
 
@@ -2627,10 +2639,10 @@ IF_NO_GC(
           bytes = [tableData bytes];
           length = [tableData length];
           /*
-           * A localisation file can be ...
-           * UTF16 with a leading BOM,
-           * UTF8 with a leading BOM,
-           * or ASCII (the original standard) with \U escapes.
+           * A localisation file can be:
+           * - UTF-16 with a leading BOM,
+           * - UTF-8,
+           * - or ASCII with \U escapes.
            */
           if (length > 2
               && ((bytes[0] == 0xFF && bytes[1] == 0xFE)
@@ -2638,30 +2650,25 @@ IF_NO_GC(
             {
               encoding = NSUnicodeStringEncoding;
             }
-          else if (length > 2
-                   && bytes[0] == 0xEF && bytes[1] == 0xBB && bytes[2] == 0xBF)
+          else
             {
               encoding = NSUTF8StringEncoding;
             }
-          else
-            {
-              encoding = NSASCIIStringEncoding;
-            }
           tableContent = [[NSString alloc] initWithData: tableData
                                            encoding: encoding];
-          if (tableContent == nil && encoding == NSASCIIStringEncoding)
+          if (tableContent == nil && encoding == NSUTF8StringEncoding)
             {
               encoding = [NSString defaultCStringEncoding];
               tableContent = [[NSString alloc] initWithData: tableData
                                                encoding: encoding];
               if (tableContent != nil)
                 {
-                  NSWarnMLog (@"Localisation file %@ not in portable encoding"
-		    @" so I'm using the default encoding for the current"
-		    @" system, which may not display messages correctly.\n"
-		    @"The file should be ASCII (using \\U escapes for unicode"
-		    @" characters) or Unicode (UTF16 or UTF8) with a leading "
-		    @"byte-order-marker.\n", tablePath);
+                  NSWarnMLog (@"Localisation file %@ not in portable encoding,"
+                    @" so I'm using the default encoding for the current"
+                    @" system, which may not display messages correctly.\n"
+                    @"The file should be UTF-8, UTF-16 with a leading"
+                    @" byte-order-marker, or ASCII (using \\U escapes for"
+                    @" unicode characters.\n", tablePath);
                 }
             }
           if (tableContent == nil)
