@@ -39,17 +39,22 @@
 #import "Foundation/NSCoder.h"
 
 #if defined(_WIN32)
-#include <winsock2.h>
+#ifdef HAVE_WS2TCPIP_H
 #include <ws2tcpip.h>
-extern const char *inet_ntop(int, const void *, char *, size_t);
-extern int inet_pton(int , const char *, void *);
-#else
+#endif // HAVE_WS2TCPIP_H
+#if !defined(HAVE_INET_NTOP)
+extern const char* WSAAPI inet_ntop(int, const void *, char *, size_t);
+#endif
+#if !defined(HAVE_INET_NTOP)
+extern int WSAAPI inet_pton(int , const char *, void *);
+#endif
+#else /* !_WIN32 */
 #include <netdb.h>
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#endif /* !_WIN32*/
+#endif /* !_WIN32 */
 
 #ifndef	INADDR_NONE
 #define	INADDR_NONE	-1
@@ -441,7 +446,7 @@ myHostName()
 	   * with ALL the IP addresses of any interfaces on the local machine
 	   */
 	  host = [[self alloc] _initWithHostEntry: 0 key: localHostName];
-	  IF_NO_GC([host autorelease];)
+	  IF_NO_ARC([host autorelease];)
 	}
       else
 	{
@@ -469,7 +474,7 @@ myHostName()
 	  else
 	    {
 	      host = [[self alloc] _initWithHostEntry: h key: name];
-	      IF_NO_GC([host autorelease];)
+	      IF_NO_ARC([host autorelease];)
 	    }
 	}
     }
@@ -479,7 +484,7 @@ myHostName()
     }
   else
     {
-      IF_NO_GC([[host retain] autorelease];)
+      IF_NO_ARC([[host retain] autorelease];)
     }
   [_hostCacheLock unlock];
   return host;
@@ -554,17 +559,17 @@ myHostName()
       if (0 == h)
 	{
 	  host = [[self alloc] _initWithAddress: address];
-	  IF_NO_GC([host autorelease];)
+	  IF_NO_ARC([host autorelease];)
 	}
       else
 	{
 	  host = [[self alloc] _initWithHostEntry: h key: address];
-	  IF_NO_GC([host autorelease];)
+	  IF_NO_ARC([host autorelease];)
 	}
     }
   else
     {
-      IF_NO_GC([[host retain] autorelease];)
+      IF_NO_ARC([[host retain] autorelease];)
     }
   [_hostCacheLock unlock];
   return host;
@@ -629,7 +634,7 @@ myHostName()
     {
       host = [NSHost currentHost];
     }
-  IF_NO_GC([host retain];)
+  IF_NO_ARC([host retain];)
   DESTROY(self);
   return host;
 }
