@@ -14,12 +14,12 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Boston, MA 02110 USA.
 
    */
 
@@ -917,7 +917,7 @@ static NSUInteger posForIndex(GSIArray array, NSUInteger index)
           if ((innerI <= lastInRange) && (innerI >= range.location))
             {
               GS_DISPATCH_SUBMIT_BLOCK(enumQueueGroup, enumQueue,
-                if (shouldStop) {return;}, return;,
+                if (shouldStop == NO) {, },
                 aBlock, innerI, &shouldStop);
             }
           if (shouldStop)
