@@ -574,7 +574,10 @@ parseArray(ParserState *state)
     {
       if (NO == [array makeImmutable])
         {
+	  id	a = array;
+
           array = [array copy];
+	  RELEASE(a);
         }
     }
   return array;
@@ -638,11 +641,13 @@ parseObject(ParserState *state)
     {
       if (NO == [dict makeImmutable])
         {
+	  id	d = dict;
+
           dict = [dict copy];
+	  RELEASE(d);
         }
     }
   return dict;
-
 }
 
 /**
@@ -655,9 +660,10 @@ parseValue(ParserState *state)
 
   if (state->error) { return nil; };
   c = consumeSpace(state);
-  //   2.1: A JSON value MUST be an object, array, number, or string, or one of the
-  //   following three literal names:
-  //            false null true
+  /*   2.1: A JSON value MUST be an object, array, number, or string,
+   *   or one of the following three literal names:
+   *   false null true
+   */
   switch (c)
     {
       case (unichar)'"':
