@@ -68,12 +68,6 @@
  * with a another generic setter.
  */
 
-NSString *const NSKeyValueChangeIndexesKey = @"indexes";
-NSString *const NSKeyValueChangeKindKey = @"kind";
-NSString *const NSKeyValueChangeNewKey = @"new";
-NSString *const NSKeyValueChangeOldKey = @"old";
-NSString *const NSKeyValueChangeNotificationIsPriorKey = @"notificationIsPrior";
-
 static NSRecursiveLock	*kvoLock = nil;
 static NSMapTable	*classTable = 0;
 static NSMapTable	*infoTable = 0;
@@ -444,6 +438,11 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
   NSString		*superName;
   NSString		*name;
 
+  if (nil == (self = [super init]))
+    {
+      return nil;
+    }
+
   if ([aClass instanceMethodForSelector: @selector(takeValue:forKey:)]
     != [NSObject instanceMethodForSelector: @selector(takeValue:forKey:)])
     {
@@ -574,7 +573,7 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
                 imp = [[GSKVOSetter class]
                   instanceMethodForSelector: @selector(setterDouble:)];
                 break;
-#if __GNUC__ > 2 && defined(_C_BOOL)
+#if defined(_C_BOOL) && (!defined(__GNUC__) || __GNUC__ > 2)
               case _C_BOOL:
                 imp = [[GSKVOSetter class]
                   instanceMethodForSelector: @selector(setterChar:)];
@@ -1298,8 +1297,13 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
             withTarget: (id)aTarget
                context: (void *)context
 {
-  NSString * remainingKeyPath;
-  NSRange dot;
+  NSString	*remainingKeyPath;
+  NSRange 	dot;
+
+  if (nil == (self = [super init]))
+    {
+      return nil;
+    }
 
   target = aTarget;
   keyPathToForward = [keyPath copy];
@@ -1485,6 +1489,7 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
   NSRange               dot;
 
   setup();
+
   [kvoLock lock];
 
   // Use the original class
@@ -1549,7 +1554,7 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
        * turn off key-value-observing for it.
        */
       object_setClass(self, [self class]);
-      IF_NO_GC(AUTORELEASE(info);)
+      IF_NO_ARC(AUTORELEASE(info);)
       [self setObservationInfo: nil];
     }
   if ([aPath rangeOfString:@"."].location != NSNotFound)
@@ -1891,7 +1896,9 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
             {
               set = [self valueForKey: aKey];
             }
-          [pathInfo->change setValue: [set mutableCopy] forKey: @"oldSet"];
+	  set = [set mutableCopy];
+          [pathInfo->change setValue: set forKey: @"oldSet"];
+	  RELEASE(set);
           [pathInfo notifyForKey: aKey ofInstance: [info instance] prior: YES];
         }
       [info unlock];
@@ -1921,7 +1928,7 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
           NSMutableSet  *oldSet;
           id            set = objects;
 
-          oldSet = [pathInfo->change valueForKey: @"oldSet"];
+          oldSet = RETAIN([pathInfo->change valueForKey: @"oldSet"]);
           if (nil == set)
             {
               set = [self valueForKey: aKey];
@@ -1937,6 +1944,7 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
                         forKey: NSKeyValueChangeKindKey];
               [pathInfo->change setValue: set
                                   forKey: NSKeyValueChangeNewKey];
+	      RELEASE(set);
             }
           else if (mutationKind == NSKeyValueMinusSetMutation
             || mutationKind == NSKeyValueIntersectSetMutation)
@@ -1964,8 +1972,10 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
                                   forKey: NSKeyValueChangeOldKey];
               [pathInfo->change setValue: new
                                   forKey: NSKeyValueChangeNewKey];
+	      RELEASE(old);
+	      RELEASE(new);
             }
-
+	  RELEASE(oldSet);
           [pathInfo notifyForKey: aKey ofInstance: [info instance] prior: NO];
         }
       if (pathInfo->recursion > 0)
@@ -2048,7 +2058,7 @@ triggerChangeNotificationsForDependentKey: (NSString*)dependentKey
   setup();
   [kvoLock lock];
   info = NSMapGet(infoTable, (void*)self);
-  IF_NO_GC(AUTORELEASE(RETAIN((id)info));)
+  IF_NO_ARC(AUTORELEASE(RETAIN((id)info));)
   [kvoLock unlock];
   return info;
 }
