@@ -66,6 +66,9 @@
 #if	defined(HAVE_UNICODE_UNUM_H)
 # include <unicode/unum.h>
 #endif
+#if defined(HAVE_ICU_H)
+# include <icu.h>
+#endif
 
 #define BUFFER_SIZE 1024
 
@@ -509,22 +512,22 @@ static NSUInteger _defaultBehavior = NSNumberFormatterBehavior10_4;
 {
   NSNumberFormatter	*o = (NSNumberFormatter*) NSCopyObject(self, 0, zone);
 
-  IF_NO_GC(RETAIN(o->_negativeFormat);)
-  IF_NO_GC(RETAIN(o->_positiveFormat);)
-  IF_NO_GC(RETAIN(o->_attributesForPositiveValues);)
-  IF_NO_GC(RETAIN(o->_attributesForNegativeValues);)
-  IF_NO_GC(RETAIN(o->_maximum);)
-  IF_NO_GC(RETAIN(o->_minimum);)
-  IF_NO_GC(RETAIN(o->_roundingBehavior);)
-  IF_NO_GC(RETAIN(o->_attributedStringForNil);)
-  IF_NO_GC(RETAIN(o->_attributedStringForNotANumber);)
-  IF_NO_GC(RETAIN(o->_attributedStringForZero);)
+  IF_NO_ARC(RETAIN(o->_negativeFormat);)
+  IF_NO_ARC(RETAIN(o->_positiveFormat);)
+  IF_NO_ARC(RETAIN(o->_attributesForPositiveValues);)
+  IF_NO_ARC(RETAIN(o->_attributesForNegativeValues);)
+  IF_NO_ARC(RETAIN(o->_maximum);)
+  IF_NO_ARC(RETAIN(o->_minimum);)
+  IF_NO_ARC(RETAIN(o->_roundingBehavior);)
+  IF_NO_ARC(RETAIN(o->_attributedStringForNil);)
+  IF_NO_ARC(RETAIN(o->_attributedStringForNotANumber);)
+  IF_NO_ARC(RETAIN(o->_attributedStringForZero);)
   if (0 != internal)
     {
       int	idx;
 
       GS_COPY_INTERNAL(o, zone)
-      IF_NO_GC(
+      IF_NO_ARC(
 	[GSIVar(o,_locale) retain];
 	for (idx = 0; idx < MAX_SYMBOLS; ++idx)
 	  {
@@ -1100,7 +1103,7 @@ static NSUInteger _defaultBehavior = NSNumberFormatterBehavior10_4;
            * don't think it matters, because we don't bother with anything
            * smaller than int for NSNumbers
 	   */
-#if __GNUC__ > 2 && defined(_C_BOOL)
+#if defined(_C_BOOL) && (!defined(__GNUC__) || __GNUC__ > 2)
           case _C_BOOL:
             STRING_FROM_NUMBER(unum_format, (int)[anObject boolValue]);
             break;
