@@ -42,7 +42,7 @@
   BOOL blocked; \
   BOOL ready; \
   NSMutableArray *dependencies; \
-  GSOperationCompletionBlock completionBlock;
+  id completionBlock;
 
 #define	GS_NSOperationQueue_IVARS \
   NSRecursiveLock	*lock; \
@@ -54,7 +54,7 @@
   BOOL			suspended; \
   NSInteger		executing; \
   NSInteger		threadCount; \
-  NSInteger		count;
+  NSInteger		maxThreads;
 
 #import "Foundation/NSOperation.h"
 #import "Foundation/NSArray.h"
@@ -75,14 +75,11 @@ static void     *isFinishedCtxt = (void*)"isFinished";
 static void     *isReadyCtxt = (void*)"isReady";
 static void     *queuePriorityCtxt = (void*)"queuePriority";
 
-/* The pool of threads for 'non-concurrent' operations in a queue.
- */
-#define	POOL	8
-
 static NSArray	*empty = nil;
 
 @interface	NSOperation (Private)
 - (void) _finish;
+- (void) _updateReadyState;
 @end
 
 @implementation NSOperation
@@ -97,7 +94,7 @@ static NSArray	*empty = nil;
 + (void) initialize
 {
   empty = [NSArray new];
-  [[NSObject leakAt: &empty] release];
+  RELEASE([NSObject leakAt: &empty]);
 }
 
 - (void) addDependency: (NSOperation *)op
@@ -197,7 +194,7 @@ static NSArray	*empty = nil;
 
 - (GSOperationCompletionBlock) completionBlock
 {
-  return internal->completionBlock;
+  return (GSOperationCompletionBlock)internal->completionBlock;
 }
 
 - (void) dealloc
@@ -217,6 +214,7 @@ static NSArray	*empty = nil;
       RELEASE(internal->dependencies);
       RELEASE(internal->cond);
       RELEASE(internal->lock);
+      RELEASE(internal->completionBlock);
       GS_DESTROY_INTERNAL(NSOperation);
     }
   [super dealloc];
@@ -296,49 +294,45 @@ static NSArray	*empty = nil;
                          change: (NSDictionary *)change
                         context: (void *)context
 {
-  [internal->lock lock];
-
-  /* We only observe isFinished changes, and we can remove self as an
-   * observer once we know the operation has finished since it can never
-   * become unfinished.
-   */
-  [object removeObserver: self forKeyPath: @"isFinished"];
+  NSOperation *op = object;
+  if (NO == [op isFinished])
+    {
+      return;
+    }
 
   if (object == self)
     {
+      [internal->lock lock];
+
+      /* We only observe isFinished changes, and we can remove self as an
+       * observer once we know the operation has finished since it can never
+       * become unfinished.
+       */
+      [object removeObserver: self forKeyPath: @"isFinished"];
+
+      /* Concurrent operations: Call completion block and set internal finished
+       * state so we don't try removing the observer again in -dealloc. */
+      if (YES == [self isConcurrent])
+        {
+          internal->finished = YES;
+          CALL_BLOCK_NO_ARGS(
+            ((GSOperationCompletionBlock)internal->completionBlock));
+        }
+
       /* We have finished and need to unlock the condition lock so that
        * any waiting thread can continue.
        */
       [internal->cond lock];
       [internal->cond unlockWithCondition: 1];
+
       [internal->lock unlock];
-      return;
     }
-
-  if (NO == internal->ready)
+  else
     {
-      NSEnumerator	*en;
-      NSOperation	*op;
-
-      /* Some dependency has finished (or been removed) ...
-       * so we need to check to see if we are now ready unless we know we are.
-       * This is protected by locks so that an update due to an observed
-       * change in one thread won't interrupt anything in another thread.
+      /* Some dependency has finished ...
        */
-      en = [internal->dependencies objectEnumerator];
-      while ((op = [en nextObject]) != nil)
-        {
-          if (NO == [op isFinished])
-	    break;
-        }
-      if (op == nil)
-	{
-          [self willChangeValueForKey: @"isReady"];
-	  internal->ready = YES;
-          [self didChangeValueForKey: @"isReady"];
-	}
+      [self _updateReadyState];
     }
-  [internal->lock unlock];
 }
 
 - (NSOperationQueuePriority) queuePriority
@@ -359,12 +353,8 @@ static NSArray	*empty = nil;
 	  if (NO == internal->ready)
 	    {
 	      /* The dependency may cause us to become ready ...
-	       * fake an observation so we can deal with that.
 	       */
-	      [self observeValueForKeyPath: @"isFinished"
-				  ofObject: op
-				    change: nil
-				   context: isFinishedCtxt];
+	      [self _updateReadyState];
 	    }
 	  [self didChangeValueForKey: @"dependencies"];
 	}
@@ -381,7 +371,7 @@ static NSArray	*empty = nil;
 
 - (void) setCompletionBlock: (GSOperationCompletionBlock)aBlock
 {
-  internal->completionBlock = aBlock;
+  ASSIGNCOPY(internal->completionBlock, (id)aBlock);
 }
 
 - (void) setQueuePriority: (NSOperationQueuePriority)pri
@@ -429,8 +419,9 @@ static NSArray	*empty = nil;
 
 - (void) start
 {
-  NSAutoreleasePool	*pool = [NSAutoreleasePool new];
-  double		prio = [NSThread  threadPriority];
+  ENTER_POOL
+
+  double	prio = [NSThread  threadPriority];
 
   AUTORELEASE(RETAIN(self));	// Make sure we exist while running.
   [internal->lock lock];
@@ -485,7 +476,7 @@ static NSArray	*empty = nil;
   NS_ENDHANDLER;
 
   [self _finish];
-  [pool release];
+  LEAVE_POOL
 }
 
 - (double) threadPriority
@@ -501,12 +492,11 @@ static NSArray	*empty = nil;
 @end
 
 @implementation	NSOperation (Private)
+/* NB code calling this method must ensure that the receiver is retained
+ * until after the method returns.
+ */
 - (void) _finish
 {
-  /* retain while finishing so that we don't get deallocated when our
-   * queue removes and releases us.
-   */
-  [self retain];
   [internal->lock lock];
   if (NO == internal->finished)
     {
@@ -525,29 +515,60 @@ static NSArray	*empty = nil;
 	  internal->finished = YES;
 	  [self didChangeValueForKey: @"isFinished"];
 	}
-      if (NULL != internal->completionBlock)
-	{
-	  CALL_BLOCK_NO_ARGS(internal->completionBlock);
-	}
+      CALL_BLOCK_NO_ARGS(
+	((GSOperationCompletionBlock)internal->completionBlock));
+    }
+  [internal->lock unlock];
+}
+
+- (void) _updateReadyState
+{
+  [internal->lock lock];
+  if (NO == internal->ready)
+    {
+      NSEnumerator	*en;
+      NSOperation	*op;
+
+      /* After a dependency has finished or was removed we need to check
+       * to see if we are now ready.
+       * This is protected by locks so that an update due to an observed
+       * change in one thread won't interrupt anything in another thread.
+       */
+      en = [internal->dependencies objectEnumerator];
+      while ((op = [en nextObject]) != nil)
+        {
+          if (NO == [op isFinished])
+            break;
+        }
+      if (op == nil)
+        {
+          [self willChangeValueForKey: @"isReady"];
+          internal->ready = YES;
+          [self didChangeValueForKey: @"isReady"];
+        }
     }
   [internal->lock unlock];
-  [self release];
 }
 
 @end
 
-	  
+
 @implementation NSBlockOperation
 
-// Initialize
-- (id) init
++ (instancetype) blockOperationWithBlock: (GSBlockOperationBlock)block
 {
-  self = [super init];
-  if(self != nil)
-    {
-      _executionBlocks = [[NSMutableArray alloc] initWithCapacity: 1];
-    }
-  return self;
+  NSBlockOperation *op = [[self alloc] init];
+
+  [op addExecutionBlock: block];
+  return AUTORELEASE(op);
+}
+
+- (void) addExecutionBlock: (GSBlockOperationBlock)block
+{
+  id	blockCopy = (id)Block_copy(block);
+
+  [_executionBlocks addObject: blockCopy];
+  RELEASE(blockCopy);
 }
 
 - (void) dealloc
@@ -556,31 +577,29 @@ static NSArray	*empty = nil;
   [super dealloc];
 }
 
-// Managing the blocks in the Operation
-+ (instancetype)blockOperationWithBlock: (GSBlockOperationBlock)block
-{
-  NSBlockOperation *op = [[self alloc] init];
-  [op addExecutionBlock: block];
-  return op;
-}
-
-- (void)addExecutionBlock: (GSBlockOperationBlock)block
-{
-  [_executionBlocks addObject: block];
-}
-
 - (NSArray *) executionBlocks
 {
   return _executionBlocks;
 }
 
+- (id) init
+{
+  self = [super init];
+  if (self != nil)
+    {
+      _executionBlocks = [[NSMutableArray alloc] initWithCapacity: 1];
+    }
+  return self;
+}
+
 - (void) main
 {
-  NSEnumerator *en = [[self executionBlocks] objectEnumerator];
+  NSEnumerator 		*en = [[self executionBlocks] objectEnumerator];
   GSBlockOperationBlock theBlock;
-  while((theBlock = [en nextObject]) != NULL)
+
+  while ((theBlock = (GSBlockOperationBlock)[en nextObject]) != NULL)
     {
-      CALL_BLOCK_NO_ARGS(theBlock);
+      CALL_NON_NULL_BLOCK_NO_ARGS(theBlock);
     }
 }
 @end
@@ -601,7 +620,7 @@ GS_PRIVATE_INTERNAL(NSOperationQueue)
                         context: (void *)context;
 @end
 
-static NSInteger	maxConcurrent = 200;	// Thread pool size
+static NSInteger	maxConcurrent = 8;	// Thread pool size
 
 static NSComparisonResult
 sortFunc(id o1, id o2, void *ctxt)
@@ -801,7 +820,7 @@ static NSOperationQueue *mainQueue = nil;
     {
       GS_CREATE_INTERNAL(NSOperationQueue);
       internal->suspended = NO;
-      internal->count = NSOperationQueueDefaultMaxConcurrentOperationCount;
+      internal->maxThreads = NSOperationQueueDefaultMaxConcurrentOperationCount;
       internal->operations = [NSMutableArray new];
       internal->starting = [NSMutableArray new];
       internal->waiting = [NSMutableArray new];
@@ -822,7 +841,7 @@ static NSOperationQueue *mainQueue = nil;
 
 - (NSInteger) maxConcurrentOperationCount
 {
-  return internal->count;
+  return internal->maxThreads;
 }
 
 - (NSString*) name
@@ -835,9 +854,9 @@ static NSOperationQueue *mainQueue = nil;
       internal->name
 	= [[NSString alloc] initWithFormat: @"NSOperation %p", self];
     }
-  s = [internal->name retain];
+  s = RETAIN(internal->name);
   [internal->lock unlock];
-  return [s autorelease];
+  return AUTORELEASE(s);
 }
 
 - (NSUInteger) operationCount
@@ -870,10 +889,10 @@ static NSOperationQueue *mainQueue = nil;
 	NSStringFromClass([self class]), NSStringFromSelector(_cmd), cnt];
     }
   [internal->lock lock];
-  if (cnt != internal->count)
+  if (cnt != internal->maxThreads)
     {
       [self willChangeValueForKey: @"maxConcurrentOperationCount"];
-      internal->count = cnt;
+      internal->maxThreads = cnt;
       [self didChangeValueForKey: @"maxConcurrentOperationCount"];
     }
   [internal->lock unlock];
@@ -887,7 +906,7 @@ static NSOperationQueue *mainQueue = nil;
   if (NO == [internal->name isEqual: s])
     {
       [self willChangeValueForKey: @"name"];
-      [internal->name release];
+      RELEASE(internal->name);
       internal->name = [s copy];
       [self didChangeValueForKey: @"name"];
     }
@@ -914,10 +933,10 @@ static NSOperationQueue *mainQueue = nil;
   [internal->lock lock];
   while ((op = [internal->operations lastObject]) != nil)
     {
-      [op retain];
+      RETAIN(op);
       [internal->lock unlock];
       [op waitUntilFinished];
-      [op release];
+      RELEASE(op);
       [internal->lock lock];
     }
   [internal->lock unlock];
@@ -938,17 +957,21 @@ static NSOperationQueue *mainQueue = nil;
    */
   if (context == isFinishedCtxt)
     {
-      [internal->lock lock];
-      internal->executing--;
-      [object removeObserver: self forKeyPath: @"isFinished"];
-      [internal->lock unlock];
-      [self willChangeValueForKey: @"operations"];
-      [self willChangeValueForKey: @"operationCount"];
-      [internal->lock lock];
-      [internal->operations removeObjectIdenticalTo: object];
-      [internal->lock unlock];
-      [self didChangeValueForKey: @"operationCount"];
-      [self didChangeValueForKey: @"operations"];
+      NSOperation *op = object;
+      if (YES == [op isFinished])
+        {
+          [internal->lock lock];
+          internal->executing--;
+          [object removeObserver: self forKeyPath: @"isFinished"];
+          [internal->lock unlock];
+          [self willChangeValueForKey: @"operations"];
+          [self willChangeValueForKey: @"operationCount"];
+          [internal->lock lock];
+          [internal->operations removeObjectIdenticalTo: object];
+          [internal->lock unlock];
+          [self didChangeValueForKey: @"operationCount"];
+          [self didChangeValueForKey: @"operations"];
+        }
     }
   else if (context == queuePriorityCtxt || context == isReadyCtxt)
     {
@@ -978,7 +1001,7 @@ static NSOperationQueue *mainQueue = nil;
 
 - (void) _thread
 {
-  NSAutoreleasePool	*pool = [NSAutoreleasePool new];
+  CREATE_AUTORELEASE_POOL(arp);
 
   [[[NSThread currentThread] threadDictionary] setObject: self
                                                   forKey: threadKey];
@@ -987,6 +1010,11 @@ static NSOperationQueue *mainQueue = nil;
       NSOperation	*op;
       NSDate		*when;
       BOOL		found;
+      /* We use a pool for each operation in case releasing the operation
+       * causes it to be deallocated, and the deallocation of the operation
+       * autoreleases something which needs to be cleaned up.
+       */
+      RECREATE_AUTORELEASE_POOL(arp);
 
       when = [[NSDate alloc] initWithTimeIntervalSinceNow: 5.0];
       found = [internal->cond lockWhenCondition: 1 beforeDate: when];
@@ -1021,11 +1049,10 @@ static NSOperationQueue *mainQueue = nil;
 	{
           NS_DURING
 	    {
-	      NSAutoreleasePool	*opPool = [NSAutoreleasePool new];
-
+	      ENTER_POOL
               [NSThread setThreadPriority: [op threadPriority]];
               [op start];
-	      RELEASE(opPool);
+	      LEAVE_POOL
 	    }
           NS_HANDLER
 	    {
@@ -1042,7 +1069,7 @@ static NSOperationQueue *mainQueue = nil;
   [internal->lock lock];
   internal->threadCount--;
   [internal->lock unlock];
-  RELEASE(pool);
+  DESTROY(arp);
   [NSThread exit];
 }
 
@@ -1096,12 +1123,21 @@ static NSOperationQueue *mainQueue = nil;
 	   * we haven't reached the pool limit.
 	   */
 	  if (0 == internal->threadCount
-	    || (pending > 0 && internal->threadCount < POOL))
+	    || (pending > 0 && internal->threadCount < max))
 	    {
 	      internal->threadCount++;
-	      [NSThread detachNewThreadSelector: @selector(_thread)
-				       toTarget: self
-				     withObject: nil];
+	      NS_DURING
+		{
+		  [NSThread detachNewThreadSelector: @selector(_thread)
+					   toTarget: self
+					 withObject: nil];
+		}
+	      NS_HANDLER
+		{
+		  NSLog(@"Failed to create thread for %@: %@",
+		    self, localException);
+		}
+	      NS_ENDHANDLER
 	    }
 	  /* Tell the thread pool that there is an operation to start.
 	   */
