@@ -1162,8 +1162,8 @@ static void InitialisePathUtilities(void)
 	  char	dummy[1024];
 	} s;
 	LPTSTR	str;
-	unichar buf[1024];
-	unichar dom[1024];
+	GSNativeChar buf[1024];
+	GSNativeChar dom[1024];
 	SID_NAME_USE use;
 	DWORD bsize = 1024;
 	DWORD ssize = 1024;
@@ -1650,7 +1650,7 @@ NSUserName(void)
       else
 	{
 	  /* The GetUserName function returns the current user name */
-	  unichar buf[1024];
+	  GSNativeChar buf[1024];
 	  DWORD n = 1024;
 
 	  if (GetUserNameW(buf, &n) != 0 && buf[0] != '\0')
@@ -1829,8 +1829,9 @@ NSFullUserName(void)
 #if defined(_WIN32)
       struct _USER_INFO_2	*userInfo;
 
-      if (NetUserGetInfo(NULL, (unichar*)[userName cStringUsingEncoding:
-	NSUnicodeStringEncoding], 2, (LPBYTE*)&userInfo) == 0)
+      if (NetUserGetInfo(NULL, (const GSNativeChar*)[userName
+        cStringUsingEncoding: NSUnicodeStringEncoding], 2,
+        (LPBYTE*)&userInfo) == 0)
 	{
 	  int	length = wcslen(userInfo->usri2_full_name);
 
@@ -1957,11 +1958,14 @@ NSTemporaryDirectory(void)
 #endif
 
 #if defined(_WIN32)
-  unichar buffer[1024];
+  GSNativeChar buffer[1024];
   if (GetTempPathW(1024, buffer))
     {
       baseTempDirName = [NSString stringWithCharacters: buffer
 						length: wcslen(buffer)];
+      // convert path to use forward slashes, which we use internally
+      baseTempDirName = [baseTempDirName stringByReplacingString: @"\\"
+        withString: @"/"];
     }
 #elif defined(__ANDROID__)
   /*
@@ -2134,9 +2138,9 @@ NSOpenStepRootDirectory(void)
   return root;
 }
 
-#if	defined(_WIN32)
-/* The developer root on a windows system (where we have an msys environment
- * set up) is the point in the filesystem where we can reference make.exe via
+#if	defined(__MINGW__)
+/* The developer root under MinGW (where we have an MSYS environment set up)
+ * is the point in the filesystem where we can reference make.exe via
  * msys/.../bin/.  That is, it's the windows path at which msys is installed.
  */
 static NSString*
@@ -2144,7 +2148,7 @@ devroot(NSFileManager *manager, NSString *path)
 {
   NSString      *tmp = @"";
 
-  while (NO == [tmp isEqual: path])
+  while (path && NO == [tmp isEqual: path])
     {
       NSString	*pb;
       NSString	*msys;
@@ -2323,7 +2327,7 @@ if (domainMask & mask) \
 
       case NSDeveloperDirectory:
 	{
-#if	defined(_WIN32)
+#if	defined(__MINGW__)
           if (nil == gnustepDeveloperDir)
             {
               NSString          *path = nil;
