@@ -681,7 +681,7 @@ deserializeFromInfo(_NSDeserializerInfo* info)
 			   * rather than simply releasing as the key may
 			   * be referred to by a cross-reference later.
 			   */
-			  IF_NO_GC(AUTORELEASE(k);)
+			  IF_NO_ARC(AUTORELEASE(k);)
 			  RELEASE(o);
 			}
 		    }
