@@ -40,17 +40,13 @@
 #import "Foundation/NSException.h"
 #import "Foundation/NSSet.h"
 
-NSString *const NSGrammarRange = @"NSGrammarRange";
-NSString *const NSGrammarUserDescription = @"NSGrammarUserDescription";
-NSString *const NSGrammarCorrections = @"NSGrammarCorrections";
-
 static NSConnection *spellServerConnection = nil;
 
 /* User dictionary location */
 static NSString *GNU_UserDictionariesDir = @"Dictionaries";
 
 // Function to create name for spell server....
-NSString*
+GS_DECLARE NSString*
 GSSpellServerName(NSString *vendor, NSString *language)
 {
   NSString *serverName = nil;
@@ -153,7 +149,7 @@ GSSpellServerName(NSString *vendor, NSString *language)
 - (void) setDelegate: (id)anObject
 {
   /* FIXME - we should not retain the delegate ! */
-  IF_NO_GC(RETAIN(anObject);)
+  IF_NO_ARC(RETAIN(anObject);)
   ASSIGN(_delegate, anObject);
 }
 
