@@ -38,12 +38,13 @@
 #import "Foundation/NSMapTable.h"
 #import "Foundation/NSProcessInfo.h"
 #import "Foundation/NSRunLoop.h"
+#import "Foundation/NSLock.h"
 #import "Foundation/NSNotification.h"
 #import "Foundation/NSNotificationQueue.h"
 #import "Foundation/NSTask.h"
 #import "Foundation/NSThread.h"
 #import "Foundation/NSTimer.h"
-#import "Foundation/NSLock.h"
+#import "Foundation/NSURL.h"
 #import "GNUstepBase/NSString+GNUstepBase.h"
 #import "GNUstepBase/NSTask+GNUstepBase.h"
 #import "GSPrivate.h"
@@ -414,7 +415,13 @@ pty_slave(const char* name)
  */
 - (void) launch
 {
-  ASSIGN(_launchingThread, [NSThread currentThread]);
+  NSError	*e;
+
+  if (NO == [self launchAndReturnError: &e])
+    {
+      [NSException raise: NSInvalidArgumentException
+		  format: @"%@", e ? e : @"Unable to launch"];
+    }
 }
 
 /**
@@ -850,7 +857,7 @@ pty_slave(const char* name)
   NSTimer	*timer = nil;
   NSDate	*limit = nil;
 
-  IF_NO_GC([[self retain] autorelease];)
+  IF_NO_ARC([[self retain] autorelease];)
   while ([self isRunning])
     {
       /* Poll at 0.1 second intervals.
@@ -876,6 +883,49 @@ pty_slave(const char* name)
   [loop runMode: NSDefaultRunLoopMode beforeDate: limit];
   LEAVE_POOL
 }
+
+// macOS 10.13 methods...
+
++ (NSTask *) launchedTaskWithExecutableURL: (NSURL *)url 
+  arguments: (NSArray *)arguments 
+  error: (NSError **)error 
+  terminationHandler: (GSTaskTerminationHandler)terminationHandler
+{
+  NSTask	*task = [self launchedTaskWithLaunchPath: [url path]
+					       arguments: arguments];
+  task->_handler = terminationHandler;
+  if (error)
+    {
+      *error = nil;
+     } 
+  return task;
+}
+
+- (BOOL) launchAndReturnError: (NSError **)error
+{
+  ASSIGN(_launchingThread, [NSThread currentThread]);
+  return YES;
+}
+
+- (NSURL *) executableURL
+{
+  return [NSURL URLWithString: [self launchPath]];;
+}
+
+- (void) setExecutableURL: (NSURL *)url
+{
+  [self setLaunchPath: [url path]];
+}
+
+- (NSURL *) currentDirectoryURL
+{
+  return [NSURL URLWithString: [self currentDirectoryPath]];
+}
+
+- (void) setCurrentDirectoryURL: (NSURL *)url
+{
+  [self setCurrentDirectoryPath: [url path]];
+}
 @end
 
 @implementation	NSTask (Private)
@@ -918,12 +968,17 @@ pty_slave(const char* name)
             postingStyle: NSPostASAP
             coalesceMask: NSNotificationNoCoalescing
                 forModes: nil];
+
+  if (_handler != nil)
+    {
+      CALL_BLOCK_NO_ARGS(_handler);
+    }
 }
 
 - (void) _terminatedChild: (int)status reason: (NSTaskTerminationReason)reason
 {
   [tasksLock lock];
-  IF_NO_GC([[self retain] autorelease];)
+  IF_NO_ARC([[self retain] autorelease];)
   NSMapRemove(activeTasks, (void*)(intptr_t)_taskId);
   [tasksLock unlock];
   _terminationStatus = status;
@@ -1125,7 +1180,8 @@ quotedFromString(NSString *aString)
     }
 }
 
-- (void) launch
+// 10.13 method...
+- (BOOL) launchAndReturnError: (NSError **)error
 {
   STARTUPINFOW		start_info;
   NSString      	*lpath;
@@ -1146,11 +1202,20 @@ quotedFromString(NSString *aString)
 
   if (_hasLaunched)
     {
-      [NSException raise: NSInvalidArgumentException
-                  format: @"NSTask - task has already been launched"];
+      if (error)
+	{
+	  NSDictionary      *info;
+
+	  info = [NSDictionary dictionaryWithObjectsAndKeys:
+	    @"task has already been launched", NSLocalizedDescriptionKey, nil];
+	  *error = [NSError errorWithDomain: NSCocoaErrorDomain
+				       code: 0
+				   userInfo: info];
+	}
+      return NO;
     }
 
-  [super launch];
+  [super launchAndReturnError: error];
 
   lpath = [self _fullLaunchPath];
   wexecutable = (const unichar*)[lpath fileSystemRepresentation];
@@ -1307,7 +1372,7 @@ quotedFromString(NSString *aString)
     (const unichar*)[[self currentDirectoryPath] fileSystemRepresentation],
     &start_info,
     &procInfo);
-  if (result == 0)
+  if (0 == result)
     {
       last = [NSError _last];
     }
@@ -1319,8 +1384,11 @@ quotedFromString(NSString *aString)
 
   if (0 == result)
     {
-      [NSException raise: NSInvalidArgumentException
-        format: @"NSTask - Error launching task: %@ ... %@", lpath, last];
+      if (error)
+	{
+	  *error = last;
+	}
+      return NO;
     }
 
   _taskId = procInfo.dwProcessId;
@@ -1347,6 +1415,8 @@ quotedFromString(NSString *aString)
       [hdl closeFile];
       [toClose removeObjectAtIndex: 0];
     }
+
+  return YES;
 }
 
 - (void) _collectChild
@@ -1397,7 +1467,7 @@ GSPrivateCheckTasks()
 #if	defined(WAITDEBUG)
 	      [tasksLock lock];
 	      t = (NSTask*)NSMapGet(activeTasks, (void*)(intptr_t)result);
-	      IF_NO_GC([[t retain] autorelease];)
+	      IF_NO_ARC([[t retain] autorelease];)
 	      [tasksLock unlock];
 	      if (t != nil)
 		{
@@ -1410,7 +1480,7 @@ GSPrivateCheckTasks()
 	    {
 	      [tasksLock lock];
 	      t = (NSTask*)NSMapGet(activeTasks, (void*)(intptr_t)result);
-	      IF_NO_GC([[t retain] autorelease];)
+	      IF_NO_ARC([[t retain] autorelease];)
 	      [tasksLock unlock];
 	      if (t != nil)
 		{
@@ -1453,7 +1523,8 @@ GSPrivateCheckTasks()
   return found;
 }
 
-- (void) launch
+// 10.13 method...
+- (BOOL) launchAndReturnError: (NSError **)error
 {
   NSMutableArray	*toClose;
   NSString      	*lpath;
@@ -1475,11 +1546,20 @@ GSPrivateCheckTasks()
 
   if (_hasLaunched)
     {
-      [NSException raise: NSInvalidArgumentException
-                  format: @"NSTask - task has already been launched"];
+      if (error)
+	{
+	  NSDictionary      *info;
+
+	  info = [NSDictionary dictionaryWithObjectsAndKeys:
+	    @"task has already been launched", NSLocalizedDescriptionKey, nil];
+	  *error = [NSError errorWithDomain: NSCocoaErrorDomain
+				       code: 0
+				   userInfo: info];
+	}
+      return NO;
     }
 
-  [super launch];
+  [super launchAndReturnError: error];
 
   lpath = [self _fullLaunchPath];
   executable = [lpath fileSystemRepresentation];
@@ -1551,8 +1631,11 @@ GSPrivateCheckTasks()
   pid = vfork();
   if (pid < 0)
     {
-      [NSException raise: NSInvalidArgumentException
-                  format: @"NSTask - failed to create child process"];
+      if (error)
+	{
+	  *error = [NSError _last];
+	}
+      return NO;
     }
   if (pid == 0)
     {
@@ -1674,6 +1757,8 @@ GSPrivateCheckTasks()
 	  [toClose removeObjectAtIndex: 0];
 	}
     }
+  
+  return YES;
 }
 
 - (void) setStandardError: (id)hdl
