@@ -110,16 +110,16 @@
 #import "GSPrivate.h"
 #import "GSPThread.h"
 
-#ifdef HAVE_TZHEAD
-#include <tzfile.h>
-#else
-#include "nstzfile.h"
-#endif
+/* In systems without POSIX time zone information we can use our own builtin
+ * time zone data.
+ */
 
 #if defined(HAVE_UNICODE_UCAL_H)
 #define id id_ucal
 #include <unicode/ucal.h>
 #undef id
+#elif defined(HAVE_ICU_H)
+#include <icu.h>
 #endif
 
 /* Key for local time zone in user defaults. */
@@ -163,6 +163,11 @@
 #define BUFFER_SIZE 512
 #define WEEK_MILLISECONDS (7.0*24.0*60.0*60.0*1000.0)
 
+/* Include public domain code (modified for use here) to parse standard
+ * posix time zone files.
+ */
+#include "tzdb.h"
+
 #if GS_USE_ICU == 1
 static inline int
 _NSToICUTZDisplayStyle(NSTimeZoneNameStyle style)
@@ -241,20 +246,15 @@ typedef struct {
   BOOL		isdst;
   unsigned char	abbr_idx;
   char		pad[2];
-  NSString	*abbreviation;
+  const char	*abbreviation;
 } TypeInfo;
 
 @interface	GSTimeZone : NSTimeZone
 {
 @public
   NSString	*timeZoneName;
-  NSArray	*abbreviations;
   NSData	*timeZoneData;
-  unsigned int	n_trans;
-  unsigned int	n_types;
-  int32_t	*trans;
-  TypeInfo	*types;
-  unsigned char	*idxs;
+  struct state  *sp;
 }
 @end
 
@@ -289,7 +289,7 @@ static NSMutableDictionary *abbreviationDictionary = nil;
 static NSMutableDictionary *abbreviationMap = nil;
 
 /* Lock for creating time zones. */
-static pthread_mutex_t zone_mutex;
+static gs_mutex_t zone_mutex;
 
 static Class	NSTimeZoneClass;
 static Class	GSPlaceholderTimeZoneClass;
@@ -387,14 +387,14 @@ static NSString *_time_zone_path(NSString *subpath, NSString *type)
    * Return a cached time zone if possible.
    * NB. if data of cached zone does not match new data ... don't use cache
    */
-  pthread_mutex_lock(&zone_mutex);
+  GS_MUTEX_LOCK(zone_mutex);
   zone = [zoneDictionary objectForKey: name];
   if (data != nil && [data isEqual: [zone data]] == NO)
     {
       zone = nil;
     }
-  IF_NO_GC(RETAIN(zone));
-  pthread_mutex_unlock(&zone_mutex);
+  IF_NO_ARC(RETAIN(zone);)
+  GS_MUTEX_UNLOCK(zone_mutex);
 
   if (zone == nil)
     {
@@ -657,13 +657,13 @@ static NSMapTable	*absolutes = 0;
 {
   if (offset != uninitialisedOffset)
     {
-      pthread_mutex_lock(&zone_mutex);
+      GS_MUTEX_LOCK(zone_mutex);
       NSMapRemove(absolutes, (void*)(uintptr_t)offset);
-      pthread_mutex_unlock(&zone_mutex);
+      GS_MUTEX_UNLOCK(zone_mutex);
     }
   RELEASE(name);
   RELEASE(detail);
-  [super dealloc];
+  DEALLOC;
 }
 
 - (void) encodeWithCoder: (NSCoder*)aCoder
@@ -709,17 +709,17 @@ static NSMapTable	*absolutes = 0;
       z = commonAbsolutes[anOffset/900 + 72];
       if (z != nil)
         {
-          IF_NO_GC(RETAIN(z));
+          IF_NO_ARC(RETAIN(z);)
           DESTROY(self);
           return z;
         }
     }
 
-  pthread_mutex_lock(&zone_mutex);
+  GS_MUTEX_LOCK(zone_mutex);
   z = (GSAbsTimeZone*)NSMapGet(absolutes, (void*)(uintptr_t)anOffset);
   if (z != nil)
     {
-      IF_NO_GC(RETAIN(z));
+      IF_NO_ARC(RETAIN(z);)
       DESTROY(self);
     }
   else
@@ -766,7 +766,7 @@ static NSMapTable	*absolutes = 0;
           commonAbsolutes[index] = RETAIN(self);
         }
     }
-  pthread_mutex_unlock(&zone_mutex);
+  GS_MUTEX_UNLOCK(zone_mutex);
   return z;
 }
 
@@ -807,7 +807,7 @@ static NSMapTable	*absolutes = 0;
 - (void) dealloc
 {
   RELEASE(timeZone);
-  [super dealloc];
+  DEALLOC
 }
 
 - (id) initWithCoder: (NSCoder*)aDecoder
@@ -883,7 +883,7 @@ static NSMapTable	*absolutes = 0;
 - (void) dealloc
 {
   RELEASE(zone);
-  [super dealloc];
+  DEALLOC
 }
 
 - (id) initWithTimeZone: (GSAbsTimeZone*)aZone
@@ -986,7 +986,7 @@ static NSMapTable	*absolutes = 0;
     {
       return abbreviationDictionary;
     }
-  pthread_mutex_lock(&zone_mutex);
+  GS_MUTEX_LOCK(zone_mutex);
   if (abbreviationDictionary == nil)
     {
       NSAutoreleasePool	*pool = [NSAutoreleasePool new];
@@ -1043,7 +1043,7 @@ static NSMapTable	*absolutes = 0;
 	}
       [pool drain];
     }
-  pthread_mutex_unlock(&zone_mutex);
+  GS_MUTEX_UNLOCK(zone_mutex);
   return abbreviationDictionary;
 }
 
@@ -1061,7 +1061,7 @@ static NSMapTable	*absolutes = 0;
     {
       return abbreviationMap;
     }
-  pthread_mutex_lock(&zone_mutex);
+  GS_MUTEX_LOCK(zone_mutex);
   if (abbreviationMap == nil)
     {
       NSAutoreleasePool		*pool = [NSAutoreleasePool new];
@@ -1095,7 +1095,7 @@ static NSMapTable	*absolutes = 0;
 #endif
 	  if (file == NULL)
 	    {
-              pthread_mutex_unlock(&zone_mutex);
+              GS_MUTEX_UNLOCK(zone_mutex);
 	      [NSException
 		raise: NSInternalInconsistencyException
 		format: @"Failed to open time zone abbreviation map."];
@@ -1186,7 +1186,7 @@ static NSMapTable	*absolutes = 0;
         }
       [pool drain];
     }
-  pthread_mutex_unlock(&zone_mutex);
+  GS_MUTEX_UNLOCK(zone_mutex);
 
   return abbreviationMap;
 }
@@ -1204,7 +1204,7 @@ static NSMapTable	*absolutes = 0;
       return namesArray;
     }
 
-  pthread_mutex_lock(&zone_mutex);
+  GS_MUTEX_LOCK(zone_mutex);
   if (namesArray == nil)
     {
       unsigned		i;
@@ -1230,7 +1230,7 @@ static NSMapTable	*absolutes = 0;
           RELEASE(ma);
         }
     }
-  pthread_mutex_unlock(&zone_mutex);
+  GS_MUTEX_UNLOCK(zone_mutex);
   return namesArray;
 }
 
@@ -1260,7 +1260,7 @@ static NSMapTable	*absolutes = 0;
 	   * locate the correct placeholder in the (lock protected)
 	   * table of placeholders.
 	   */
-          pthread_mutex_lock(&zone_mutex);
+          GS_MUTEX_LOCK(zone_mutex);
 	  obj = (id)NSMapGet(placeholderMap, (void*)z);
 	  if (obj == nil)
 	    {
@@ -1271,7 +1271,7 @@ static NSMapTable	*absolutes = 0;
 	      obj = (id)NSAllocateObject(GSPlaceholderTimeZoneClass, 0, z);
 	      NSMapInsert(placeholderMap, (void*)z, (void*)obj);
 	    }
-          pthread_mutex_unlock(&zone_mutex);
+          GS_MUTEX_UNLOCK(zone_mutex);
 	  return obj;
 	}
     }
@@ -1288,7 +1288,7 @@ static NSMapTable	*absolutes = 0;
 {
   NSTimeZone	*zone;
 
-  pthread_mutex_lock(&zone_mutex);
+  GS_MUTEX_LOCK(zone_mutex);
   if (defaultTimeZone == nil)
     {
       zone = [self systemTimeZone];
@@ -1297,7 +1297,7 @@ static NSMapTable	*absolutes = 0;
     {
       zone = AUTORELEASE(RETAIN(defaultTimeZone));
     }
-  pthread_mutex_unlock(&zone_mutex);
+  GS_MUTEX_UNLOCK(zone_mutex);
   return zone;
 }
 
@@ -1306,7 +1306,7 @@ static NSMapTable	*absolutes = 0;
   if (self == [NSTimeZone class])
     {
       NSTimeZoneClass = self;
-      GS_INIT_RECURSIVE_MUTEX(zone_mutex);
+      GS_MUTEX_INIT_RECURSIVE(zone_mutex);
       GSPlaceholderTimeZoneClass = [GSPlaceholderTimeZone class];
       zoneDictionary = [[NSMutableDictionary alloc] init];
       [[NSObject leakAt: &zoneDictionary] release];
@@ -1351,9 +1351,9 @@ static NSMapTable	*absolutes = 0;
  */
 + (void) resetSystemTimeZone
 {
-  pthread_mutex_lock(&zone_mutex);
+  GS_MUTEX_LOCK(zone_mutex);
   DESTROY(systemTimeZone);
-  pthread_mutex_unlock(&zone_mutex);
+  GS_MUTEX_UNLOCK(zone_mutex);
   [[NSNotificationCenter defaultCenter]
     postNotificationName: NSSystemTimeZoneDidChangeNotification
                   object: nil];
@@ -1374,9 +1374,9 @@ static NSMapTable	*absolutes = 0;
 	{
 	  aTimeZone = [self systemTimeZone];
 	}
-      pthread_mutex_lock(&zone_mutex);
+      GS_MUTEX_LOCK(zone_mutex);
       ASSIGN(defaultTimeZone, aTimeZone);
-      pthread_mutex_unlock(&zone_mutex);
+      GS_MUTEX_UNLOCK(zone_mutex);
     }
 }
 
@@ -1387,7 +1387,7 @@ static NSMapTable	*absolutes = 0;
 {
   NSTimeZone	*zone = nil;
 
-  pthread_mutex_lock(&zone_mutex);
+  GS_MUTEX_LOCK(zone_mutex);
   if (systemTimeZone == nil)
     {
       NSFileManager *dflt = [NSFileManager defaultManager];
@@ -1437,20 +1437,60 @@ static NSMapTable	*absolutes = 0;
        * Try to get timezone from windows system call.
        */
       {
-      	TIME_ZONE_INFORMATION tz;
-      	DWORD DST = GetTimeZoneInformation(&tz);
+        TIME_ZONE_INFORMATION tz;
+        DWORD dst;
+        wchar_t *tzName;
+
+#if defined(_MSC_VER) && defined(UCAL_H)
+        // Get time zone name for US locale as expected by ICU method below
+        LANGID origLangID = GetThreadUILanguage();
+        SetThreadUILanguage(MAKELANGID(LANG_ENGLISH, SUBLANG_ENGLISH_US));
+        dst = GetTimeZoneInformation(&tz);
+        SetThreadUILanguage(origLangID);
+
+        // Only tz.StandardName time zone conversions are supported, as
+        // the Zone-Tzid table lacks all daylight time conversions:
+        // e.g. 'W. Europe Daylight Time' <-> 'Europe/Berlin' is not listed.
+        //
+        // See: https://unicode-org.github.io/cldr-staging/charts/latest/supplemental/zone_tzid.html
+        tzName = tz.StandardName;
+#else
+        dst = GetTimeZoneInformation(&tz);
+
+        if (dst == TIME_ZONE_ID_DAYLIGHT)
+          tzName = tz.DaylightName;
+        else
+          tzName = tz.StandardName;
+#endif
 
         localZoneSource = @"function: 'GetTimeZoneInformation()'";
-      	if (DST == TIME_ZONE_ID_DAYLIGHT)
-	  {
-	    localZoneString = [NSString stringWithCharacters: tz.DaylightName
-	      length: wcslen(tz.DaylightName)];
-	  }
-      	else
-	  {
-	    localZoneString = [NSString stringWithCharacters: tz.StandardName
-	      length: wcslen(tz.StandardName)];
-	  }
+
+#if defined(_MSC_VER) && defined(UCAL_H)
+        // Convert Windows timezone name to IANA identifier
+        if (tzName)
+          {
+            UErrorCode status = U_ZERO_ERROR;
+            UChar ianaTzName[128];
+            int32_t ianaTzNameLen = ucal_getTimeZoneIDForWindowsID(tzName,
+              -1, NULL, ianaTzName, 128, &status);
+            if (U_SUCCESS(status) && ianaTzNameLen > 0) {
+              localZoneString = [NSString stringWithCharacters: ianaTzName
+                length: ianaTzNameLen];
+            } else if (U_SUCCESS(status)) {
+              // this happens when ICU has no mapping for the time zone
+              NSLog(@"Unable to map timezone '%ls' to IANA format", tzName);
+            } else {
+              NSLog(@"Error converting timezone '%ls' to IANA format: %s",
+                tzName, u_errorName(status));
+            }
+          }
+#endif
+
+        if (localZoneString == nil)
+          {
+            localZoneString = [NSString stringWithCharacters: tzName
+              length: wcslen(tzName)];
+          }
       }
 #endif
 
@@ -1727,7 +1767,7 @@ localZoneString, [zone name], sign, s/3600, (s/60)%60);
       ASSIGN(systemTimeZone, zone);
     }
   zone = AUTORELEASE(RETAIN(systemTimeZone));
-  pthread_mutex_unlock(&zone_mutex);
+  GS_MUTEX_UNLOCK(zone_mutex);
   return zone;
 }
 
@@ -1746,7 +1786,7 @@ localZoneString, [zone name], sign, s/3600, (s/60)%60);
     {
       return regionsArray;
     }
-  pthread_mutex_lock(&zone_mutex);
+  GS_MUTEX_LOCK(zone_mutex);
   if (regionsArray == nil)
     {
       NSAutoreleasePool	*pool = [NSAutoreleasePool new];
@@ -1778,7 +1818,7 @@ localZoneString, [zone name], sign, s/3600, (s/60)%60);
 #endif
 	  if (fp == NULL)
 	    {
-              pthread_mutex_unlock(&zone_mutex);
+              GS_MUTEX_UNLOCK(zone_mutex);
 	      [NSException
 		raise: NSInternalInconsistencyException
 		format: @"Failed to open time zone regions array file."];
@@ -1827,7 +1867,7 @@ localZoneString, [zone name], sign, s/3600, (s/60)%60);
 			     to be in this directory, but initWithName:data:
 			     will do this anyway and log a message if not. */
 			  zone = [[self alloc] initWithName: name data: data];
-			  IF_NO_GC([zone autorelease];)
+			  IF_NO_ARC([zone autorelease];)
 			}
 		      if (zone != nil)
 			{
@@ -1878,7 +1918,7 @@ localZoneString, [zone name], sign, s/3600, (s/60)%60);
       regionsArray = [[NSArray alloc] initWithObjects: temp_array count: 24];
       [pool drain];
     }
-  pthread_mutex_unlock(&zone_mutex);
+  GS_MUTEX_UNLOCK(zone_mutex);
   return regionsArray;
 }
 
@@ -1921,9 +1961,9 @@ localZoneString, [zone name], sign, s/3600, (s/60)%60);
     }
   else
     {
-      pthread_mutex_lock(&zone_mutex);
+      GS_MUTEX_LOCK(zone_mutex);
       zone = (NSTimeZone*)NSMapGet(absolutes, (void*)(uintptr_t)seconds);
-      pthread_mutex_unlock(&zone_mutex);
+      GS_MUTEX_UNLOCK(zone_mutex);
     }
   if (nil == zone)
     {
@@ -2296,8 +2336,8 @@ localZoneString, [zone name], sign, s/3600, (s/60)%60);
   return [self nextDaylightSavingTimeTransitionAfterDate: [NSDate date]];
 }
 
-- (NSString *)localizedName: (NSTimeZoneNameStyle)style
-                     locale: (NSLocale *)locale
+- (NSString *) localizedName: (NSTimeZoneNameStyle)style
+                      locale: (NSLocale *)locale
 {
 #if GS_USE_ICU == 1
   UChar *result;
@@ -2403,7 +2443,7 @@ static NSString *zoneDirs[] = {
 
   if (beenHere == NO && tzdir == nil)
     {
-      pthread_mutex_lock(&zone_mutex);
+      GS_MUTEX_LOCK(zone_mutex);
       if (beenHere == NO && tzdir == nil)
 	{
 	  NSFileManager	*mgr = [NSFileManager defaultManager];
@@ -2424,7 +2464,7 @@ static NSString *zoneDirs[] = {
 	    }
 	  beenHere = YES;
 	}
-      pthread_mutex_unlock(&zone_mutex);
+      GS_MUTEX_UNLOCK(zone_mutex);
     }
   /* Use the system zone info if possible, otherwise, use our installed
      info.  */
@@ -2514,7 +2554,7 @@ GSBreakTime(NSTimeInterval when,
   RELEASE(daylightZoneName);
   RELEASE(timeZoneNameAbbr);
   RELEASE(daylightZoneNameAbbr);
-  [super dealloc];
+  DEALLOC
 }
 
 - (id) initWithName: (NSString*)name data: (NSData*)data
@@ -2903,87 +2943,56 @@ GSBreakTime(NSTimeInterval when,
 
 @implementation	GSTimeZone
 
-/**
- * Perform a binary search of a transitions table to locate the index
- * of the transition to use for a particular time interval since 1970.<br />
- * We locate the index of the highest transition before the date, or zero
- * if there is no transition before it.
- */
-static TypeInfo*
-chop(NSTimeInterval since, GSTimeZone *zone)
-{
-  int32_t		when = (int32_t)since;
-  int32_t		*trans = zone->trans;
-  unsigned		hi = zone->n_trans;
-  unsigned		lo = 0;
-  unsigned int		i;
-
-  if (hi == 0 || trans[0] > when)
-    {
-      unsigned	n_types = zone->n_types;
-
-      /*
-       * If the first transition is greater than our date,
-       * we locate the first non-DST transition and use that offset,
-       * or just use the first transition.
-       */
-      for (i = 0; i < n_types; i++)
-	{
-	  if (zone->types[i].isdst == 0)
-	    {
-	      return &zone->types[i];
-	    }
-	}
-      return &zone->types[0];
-    }
-  else
-    {
-      for (i = hi/2; hi != lo; i = (hi + lo)/2)
-	{
-	  if (when < trans[i])
-	    {
-	      hi = i;
-	    }
-	  else if (when > trans[i])
-	    {
-	      lo = ++i;
-	    }
-	  else
-	    {
-	      break;
-	    }
-	}
-      /*
-       * If we went off the top of the table or the closest transition
-       * was later than our date, we step back to find the last
-       * transition before our date.
-       */
-      if (i > 0 && (i == zone->n_trans || trans[i] > when))
-	{
-	  i--;
-	}
-      return &zone->types[zone->idxs[i]];
-    }
-}
-
 static NSTimeZoneDetail*
 newDetailInZoneForType(GSTimeZone *zone, TypeInfo *type)
-{
-  GSTimeZoneDetail	*detail;
+{ 
+  GSTimeZoneDetail      *detail;
+  NSString		*abbr;
 
-  detail = [GSTimeZoneDetail alloc];
-  detail = [detail initWithTimeZone: zone
-			 withAbbrev: type->abbreviation
-			 withOffset: type->offset
-			    withDST: type->isdst];
+  abbr = [[NSString alloc] initWithUTF8String: type->abbreviation];
+  
+  detail = [[GSTimeZoneDetail alloc] initWithTimeZone: zone
+					   withAbbrev: abbr
+					   withOffset: type->offset
+					      withDST: type->isdst];
+  RELEASE(abbr);
   return detail;
 }
 
+/**
+ * Obtain TypeInfo from transisions or TZstring
+ */
+static TypeInfo
+getTypeInfo(NSTimeInterval since, GSTimeZone *zone)
+{
+  int64_t       when = (int64_t)since;
+  gstm		tm;
+  TypeInfo      type;
+
+  if (localsub(zone->sp, &when, 0, &tm) == NULL)
+    {
+      memset(&type, '\0', sizeof(type));
+    }
+  else
+    {
+      type.offset = tm.tm_gmtoff;
+      type.isdst = tm.tm_isdst;
+      type.abbr_idx = 0;
+      type.abbreviation = tm.tm_zone;
+    }
+  return type;
+}
+
+
 - (NSString*) abbreviationForDate: (NSDate*)aDate
 {
-  TypeInfo	*type = chop([aDate timeIntervalSince1970], self);
+  TypeInfo	type = getTypeInfo([aDate timeIntervalSince1970], self);
 
-  return type->abbreviation;
+  if (NULL == type.abbreviation)
+    {
+      return nil;
+    }
+  return [NSString stringWithUTF8String: type.abbreviation];
 }
 
 - (NSData*) data
@@ -2995,178 +3004,84 @@ newDetailInZoneForType(GSTimeZone *zone, TypeInfo *type)
 {
   RELEASE(timeZoneName);
   RELEASE(timeZoneData);
-  RELEASE(abbreviations);
-  if (types != 0)
+  if (sp != 0)
     {
-      NSZoneFree(NSDefaultMallocZone(), types);
+      free(sp);
     }
-  [super dealloc];
+  DEALLOC
 }
 
 - (id) initWithName: (NSString*)name data: (NSData*)data
 {
   static NSString	*fileException = @"GSTimeZoneFileException";
+  union local_storage	*lsp;
+  const char      	*zoneName;
+
+  /* The placeholder class should have dealt with loading the data
+   */
+  NSAssert([data isKindOfClass: [NSData class]], NSInvalidArgumentException);
+  NSAssert([name isKindOfClass: [NSString class]], NSInvalidArgumentException);
 
   timeZoneName = [name copy];
   timeZoneData = [data copy];
+
+  zoneName = [timeZoneName UTF8String];
+  sp = malloc(sizeof(*sp));
+  memset((char *)sp, '\0', sizeof(*sp));
+
   NS_DURING
     {
-      const void	*bytes = [timeZoneData bytes];
-      unsigned		length = [timeZoneData length];
-      void		*buf;
-      unsigned		pos = 0;
-      unsigned		i, charcnt;
-      unsigned char	*abbr;
-      struct tzhead	*header;
+      size_t		nread;
+      union input_buffer  	*up;
 
-      if (length < sizeof(struct tzhead))
+      lsp = malloc(sizeof(*lsp));
+      up = &lsp->u.u;
+      nread = [data length];
+      if (nread > sizeof(up->buf))
 	{
 	  [NSException raise: fileException
-		      format: @"File is too small"];
+		      format: @"data too large"];
 	}
-      header = (struct tzhead *)(bytes + pos);
-      pos += sizeof(struct tzhead);
-#ifdef TZ_MAGIC
-      if (memcmp(header->tzh_magic, TZ_MAGIC, strlen(TZ_MAGIC)) != 0)
+      [data getBytes: up->buf]; 
+
+      if (tzloadbody1(zoneName, sp, true, lsp, nread) != 0)
 	{
 	  [NSException raise: fileException
-		      format: @"TZ_MAGIC is incorrect"];
+		      format: @"cannot load data"];
 	}
-#endif
-      n_trans = GSSwapBigI32ToHost(*(int32_t*)(void*)header->tzh_timecnt);
-      n_types = GSSwapBigI32ToHost(*(int32_t*)(void*)header->tzh_typecnt);
-      charcnt = GSSwapBigI32ToHost(*(int32_t*)(void*)header->tzh_charcnt);
+      free(lsp);
+      lsp = NULL;
 
-      i = pos;
-      i += sizeof(int32_t)*n_trans;
-      if (i > length)
-	{
-	  [NSException raise: fileException
-		      format: @"Transitions list is truncated"];
-	}
-      i += n_trans;
-      if (i > length)
-	{
-	  [NSException raise: fileException
-		      format: @"Transition indexes are truncated"];
-	}
-      i += sizeof(struct ttinfo)*n_types;
-      if (i > length)
-	{
-	  [NSException raise: fileException
-		      format: @"Types list is truncated"];
-	}
-      if (i + charcnt > length)
-	{
-	  [NSException raise: fileException
-		      format: @"Abbreviations list is truncated"];
-	}
-
-      /*
-       * Now calculate size we need to store the information
-       * for efficient access ... not the same saze as the data
-       * we received.
-       */
-      i = n_trans * (sizeof(int32_t)+1) + n_types * sizeof(TypeInfo);
-      buf = NSZoneMalloc(NSDefaultMallocZone(), i);
-      types = (TypeInfo*)buf;
-      buf += (n_types * sizeof(TypeInfo));
-      trans = (int32_t*)buf;
-      buf += (n_trans * sizeof(int32_t));
-      idxs = (unsigned char*)buf;
-
-      /* Read in transitions. */
-      for (i = 0; i < n_trans; i++)
-	{
-	  trans[i] = GSSwapBigI32ToHost(*(int32_t*)(bytes + pos));
-	  pos += sizeof(int32_t);
-	}
-      for (i = 0; i < n_trans; i++)
-	{
-	  idxs[i] = *(unsigned char*)(bytes + pos);
-	  pos++;
-	}
-      for (i = 0; i < n_types; i++)
-	{
-	  struct ttinfo	*ptr = (struct ttinfo*)(bytes + pos);
-          uint32_t      off;
-
-	  types[i].isdst = (ptr->isdst != 0 ? YES : NO);
-	  types[i].abbr_idx = ptr->abbr_idx;
-          memcpy(&off, ptr->offset, 4);
-	  types[i].offset = GSSwapBigI32ToHost(off);
-	  pos += sizeof(struct ttinfo);
-	}
-      abbr = (unsigned char*)(bytes + pos);
-      {
-	id		abbrevs[charcnt];
-	unsigned	count = 0;
-	unsigned	used = 0;
-
-	memset(abbrevs, '\0', sizeof(id)*charcnt);
-	for (i = 0; i < n_types; i++)
-	  {
-	    int	loc = types[i].abbr_idx;
-
-	    if (abbrevs[loc] == nil)
-	      {
-		abbrevs[loc]
-		  = [[NSString alloc] initWithUTF8String: (char*)abbr + loc];
-		count++;
-	      }
-	    types[i].abbreviation = abbrevs[loc];
-	  }
-	/*
-	 * Now we have created all the abbreviations, we put them in an
-	 * array for easy access later and easy deallocation if/when
-	 * the receiver is deallocated.
-	 */
-	i = charcnt;
-	while (i-- > count)
-	  {
-	    if (abbrevs[i] != nil)
-	      {
-		while (abbrevs[used] != nil)
-		  {
-		    used++;
-		  }
-		abbrevs[used] = abbrevs[i];
-		abbrevs[i] = nil;
-		if (++used >= count)
-		  {
-		    break;
-		  }
-	      }
-	  }
-	abbreviations = [[NSArray alloc] initWithObjects: abbrevs count: count];
-	while (count-- > 0)
-	  {
-	    RELEASE(abbrevs[count]);
-	  }
-      }
-
-      pthread_mutex_lock(&zone_mutex);
-      [zoneDictionary setObject: self forKey: timeZoneName];
-      pthread_mutex_unlock(&zone_mutex);
+      scrub_abbrs(sp);
     }
   NS_HANDLER
     {
+      if (lsp)
+	{
+	  free(lsp);
+	}
       DESTROY(self);
-      NSLog(@"Unable to obtain time zone `%@'... %@", name, localException);
+      NSLog(@"Unable to obtain time zone `%@'... %@",
+	name, localException);
       if ([localException name] != fileException)
 	{
 	  [localException raise];
 	}
+      return nil;
     }
   NS_ENDHANDLER
+
+  GS_MUTEX_LOCK(zone_mutex);
+  [zoneDictionary setObject: self forKey: timeZoneName];
+  GS_MUTEX_UNLOCK(zone_mutex);
   return self;
 }
 
 - (BOOL) isDaylightSavingTimeForDate: (NSDate*)aDate
 {
-  TypeInfo	*type = chop([aDate timeIntervalSince1970], self);
+  TypeInfo	type = getTypeInfo([aDate timeIntervalSince1970], self);
 
-  return type->isdst;
+  return type.isdst;
 }
 
 - (NSString*) name
@@ -3176,23 +3091,31 @@ newDetailInZoneForType(GSTimeZone *zone, TypeInfo *type)
 
 - (NSInteger) secondsFromGMTForDate: (NSDate*)aDate
 {
-  TypeInfo	*type = chop([aDate timeIntervalSince1970], self);
+  TypeInfo	type = getTypeInfo([aDate timeIntervalSince1970], self);
 
-  return type->offset;
+  return type.offset;
 }
 
 - (NSArray*) timeZoneDetailArray
 {
-  NSTimeZoneDetail	*details[n_types];
+  NSTimeZoneDetail	*details[sp->typecnt];
   unsigned		i;
   NSArray		*array;
 
-  for (i = 0; i < n_types; i++)
+  for (i = 0; i < sp->typecnt; i++)
     {
-      details[i] = newDetailInZoneForType(self, &types[i]);
+      const struct _ttinfo *const ttisp = &sp->ttis[i];
+      const char *abbr = &sp->chars[ttisp->tt_desigidx];
+      TypeInfo	type;
+
+      type.offset = ttisp->tt_utoff;
+      type.isdst = ttisp->tt_isdst;
+      type.abbr_idx = ttisp->tt_desigidx;
+      type.abbreviation = abbr;
+      details[i] = newDetailInZoneForType(self, &type);
     }
-  array = [NSArray arrayWithObjects: details count: n_types];
-  for (i = 0; i < n_types; i++)
+  array = [NSArray arrayWithObjects: details count: sp->typecnt];
+  for (i = 0; i < sp->typecnt; i++)
     {
       RELEASE(details[i]);
     }
@@ -3201,11 +3124,11 @@ newDetailInZoneForType(GSTimeZone *zone, TypeInfo *type)
 
 - (NSTimeZoneDetail*) timeZoneDetailForDate: (NSDate*)aDate
 {
-  TypeInfo		*type;
+  TypeInfo		type;
   NSTimeZoneDetail	*detail;
 
-  type = chop([aDate timeIntervalSince1970], self);
-  detail = newDetailInZoneForType(self, type);
+  type = getTypeInfo([aDate timeIntervalSince1970], self);
+  detail = newDetailInZoneForType(self, &type);
   return AUTORELEASE(detail);
 }
 
