@@ -150,7 +150,7 @@ static Class	NSDate_class;
                           repeats: (BOOL)repeats
                             block: (GSTimerBlock)block
 {
-  ASSIGN(_block, block);
+  ASSIGN(_block, (id)block);
   return [self initWithFireDate: date
                        interval: interval
                          target: nil
@@ -197,6 +197,16 @@ static Class	NSDate_class;
 					    repeats: f]);
 }
 
++ (NSTimer*) timerWithTimeInterval: (NSTimeInterval)ti
+			   repeats: (BOOL)f
+			     block: (GSTimerBlock)block
+{
+  return AUTORELEASE([[self alloc] initWithFireDate: nil
+					   interval: ti
+					    repeats: f
+					      block: block]);
+}
+
 /**
  * Create a timer which will fire after ti seconds and, if f is YES,
  * every ti seconds thereafter. On firing, invocation will be performed.<br />
@@ -280,7 +290,7 @@ static Class	NSDate_class;
     {
       if ((id)_block != nil)
         {
-          CALL_BLOCK(_block, self);
+          CALL_NON_NULL_BLOCK(_block, self);
         }
       else
         {
