@@ -157,6 +157,11 @@ typedef struct
   return AUTORELEASE(o);
 }
 
+- (void) start
+{
+  [this->_protocol startLoading];
+}
+
 - (void) cancel
 {
   [this->_protocol stopLoading];
@@ -164,6 +169,30 @@ typedef struct
   DESTROY(this->_delegate);
 }
 
+- (id) delegate
+{
+  return this->_delegate;
+}
+
+- (void) scheduleInRunLoop: (NSRunLoop *)aRunLoop 
+                   forMode: (NSRunLoopMode)mode
+{
+  NSArray *modes = [NSArray arrayWithObject: mode];
+  [aRunLoop performSelector: @selector(start)
+                     target: self
+                   argument: nil
+                      order: 0
+                      modes: modes];
+}
+
+- (void) unscheduleFromRunLoop: (NSRunLoop *)aRunLoop 
+                       forMode: (NSRunLoopMode)mode
+{
+  [aRunLoop cancelPerformSelector: @selector(start)
+                           target: self
+                         argument: nil];
+}
+
 - (void) dealloc
 {
   if (this != 0)
@@ -185,7 +214,9 @@ typedef struct
     }
 }
 
-- (id) initWithRequest: (NSURLRequest *)request delegate: (id)delegate
+- (id) initWithRequest: (NSURLRequest *)request
+              delegate: (id)delegate
+      startImmediately: (BOOL)startImmediately
 {
   if ((self = [super init]) != nil)
     {
@@ -226,12 +257,22 @@ typedef struct
 	initWithRequest: this->_request
 	cachedResponse: nil
 	client: (id<NSURLProtocolClient>)self];
-      [this->_protocol startLoading];
+      if (startImmediately == YES)
+        {
+          [this->_protocol startLoading];
+        }
       this->_debug = GSDebugSet(@"NSURLConnection");
     }
   return self;
 }
 
+- (id) initWithRequest: (NSURLRequest *)request delegate: (id)delegate
+{
+  return [self initWithRequest: request
+                      delegate: delegate
+              startImmediately: YES];
+}
+
 @end
 
 
@@ -362,6 +403,9 @@ typedef struct
                   *error = [[[collector error] retain] autorelease];
                 }
             }
+	  /* Cancel to prevent the NSURLProtocol instance retaining us.
+	   */
+	  [conn cancel];
           [conn release];
         }
       [collector release];
