@@ -38,9 +38,6 @@
 #import "Foundation/NSString.h"
 #import "Foundation/NSTimeZone.h"
 
-NSString * const NSUserNotificationDefaultSoundName
-  = @"NSUserNotificationDefaultSoundName";
-
 @interface NSUserNotification ()
 @property (readwrite) NSDate *actualDeliveryDate;
 @property (readwrite, getter=isPresented) BOOL presented;
