@@ -363,6 +363,8 @@ GS_DECLARE NSString* const GSTLSCertificateFile = @"GSTLSCertificateFile";
 GS_DECLARE NSString* const GSTLSCertificateKeyFile = @"GSTLSCertificateKeyFile";
 GS_DECLARE NSString* const GSTLSCertificateKeyPassword = @"GSTLSCertificateKeyPassword";
 GS_DECLARE NSString* const GSTLSDebug = @"GSTLSDebug";
+GS_DECLARE NSString* const GSTLSIssuers = @"GSTLSIssuers";
+GS_DECLARE NSString* const GSTLSOwners = @"GSTLSOwners";
 GS_DECLARE NSString* const GSTLSPriority = @"GSTLSPriority";
 GS_DECLARE NSString* const GSTLSRemoteHosts = @"GSTLSRemoteHosts";
 GS_DECLARE NSString* const GSTLSRevokeFile = @"GSTLSRevokeFile";
