@@ -11,10 +11,11 @@ START_SET("TLS support")
 #ifndef HAVE_GNUTLS_X509_PRIVKEY_IMPORT2
  testHopeful = YES;
 #endif
-  GSTLSPrivateKey        *k;
-  GSTLSCertificateList   *c;
-  NSDateFormatter        *dateFormatter;
-  NSDate                 *expiresAt;
+  GSTLSPrivateKey       *k;
+  GSTLSCertificateList  *c;
+  GSTLSCredentials	*cred;
+  NSDateFormatter       *dateFormatter;
+  NSDate                *expiresAt;
 
   k = [GSTLSPrivateKey keyFromFile: @"test.key" withPassword: @"asdf"]; 
   PASS(k != nil, "OpenSSL encrypted key can be loaded");
@@ -35,10 +36,15 @@ START_SET("TLS support")
 #endif
   [dateFormatter release];
   PASS_EQUAL([c expiresAt], expiresAt,
-    "Expiration for entire list is that of the single item")
+    "Expiration for entire list is that of the single item");
+  
+  cred = [GSTLSCredentials selfSigned: YES];
+  NSLog(@"%@", cred);
+  PASS(cred != nil, "generates self signed certificate");
 #else
   SKIP("TLS support disabled");
 #endif
+  
   END_SET("TLS support");
   DESTROY(arp);
   return 0;
