@@ -32,7 +32,19 @@ static BOOL taskTerminationNotificationReceived;
 - (void) testNSTaskNotifications
 {
   NSDate        *deadline;
+  NSString      *testsleep;
+  NSString      *testecho;
   BOOL          earlyTermination = NO;
+  
+  /* Windows MSVC adds the '.exe' suffix to executables
+   */
+#if defined(_MSC_VER)
+  testecho = @"testecho.exe";
+  testsleep = @"testsleep.exe";
+#else
+  testecho = @"testecho";
+  testsleep = @"testsleep";
+#endif
 
   for (;;)
     {
@@ -48,7 +60,7 @@ static BOOL taskTerminationNotificationReceived;
         {
           BOOL  terminated = NO;
           [task setLaunchPath:
-            [path stringByAppendingPathComponent: @"testsleep"]];
+            [path stringByAppendingPathComponent: testsleep]];
           [task launch];
           NSLog(@"Launched pid %d", [task processIdentifier]);
           NSLog(@"Running run loop for 5 seconds");
@@ -74,7 +86,7 @@ static BOOL taskTerminationNotificationReceived;
       else
         {
           [task setLaunchPath:
-            [path stringByAppendingPathComponent: @"testecho"]];
+            [path stringByAppendingPathComponent: testecho]];
           [task launch];
           NSLog(@"Launched pid %d", [task processIdentifier]);
           NSLog(@"Running run loop for 15 seconds");
