@@ -14,6 +14,18 @@ int main(int argc, char **argv, char **env)
   NSBundle *bundle;
   BOOL loaded;
   NSString *helperPath;
+  
+  /* The following test cases depend on the GSInetServerStream
+   * class which is completely broken on Windows.
+   *
+   * See: https://github.com/gnustep/libs-base/issues/266
+   *
+   * We will mark the test cases as hopeful on Windows.
+   */
+#if defined(_WIN32)
+  NSLog(@"Marking local web server tests as hopeful because GSInetServerStream is broken on Windows");
+  testHopeful = YES;
+#endif
 
   // load the test suite's classes
   fm = [NSFileManager defaultManager];
@@ -44,7 +56,7 @@ int main(int argc, char **argv, char **env)
 	mode: NO
         extra: d];
       [server setDebug: debug];
-      [server start: d]; // 127.0.0.1:1234 HTTPS
+      [server start: d]; // localhost:1234 HTTPS
 
       /* Simple GET via HTTPS without authorization with empty response's
        * body and the response's status code 204 (by default)
@@ -180,6 +192,9 @@ int main(int argc, char **argv, char **env)
 		  format: @"can't load bundle TestConnection"];
     }
 
+#if defined(_WIN32)
+  testHopeful = NO;
+#endif
 
   DESTROY(arp);
 
