@@ -20,6 +20,9 @@
 #import	"common.h"
 
 #include <stdio.h>
+#ifdef _WIN32
+#include <io.h>
+#endif
 
 #import	"Foundation/NSArray.h"
 #import	"Foundation/NSAutoreleasePool.h"
@@ -1199,7 +1202,7 @@ main(int argc, char** argv, char** env)
 #endif
 	signal(sym, ihandler);
       }
-#ifndef __MINGW__
+#ifndef _WIN32
     signal(SIGPIPE, SIG_IGN);
     signal(SIGTTOU, SIG_IGN);
     signal(SIGTTIN, SIG_IGN);
